@@ -85,10 +85,14 @@ netsnmp_subtree_free(netsnmp_subtree *a)
       SNMP_FREE(a->variables);
     }
     SNMP_FREE(a->name_a);
+    a->namelen = 0;
     SNMP_FREE(a->start_a);
+    a->start_len = 0;
     SNMP_FREE(a->end_a);
+    a->end_len = 0;
     SNMP_FREE(a->label_a);
     netsnmp_handler_registration_free(a->reginfo);
+    a->reginfo = NULL;
     SNMP_FREE(a);
   }
 }
@@ -528,7 +532,7 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
 	
 	    if (next && (next->namelen  == new_sub->namelen) &&
 		(next->priority == new_sub->priority)) {
-                netsnmp_assert("registration" != "duplicate");
+                netsnmp_assert(!"registration != duplicate"); /* always false */
 		return MIB_DUPLICATE_REGISTRATION;
 	    }
 
@@ -622,7 +626,7 @@ netsnmp_register_mib(const char *moduleName,
         snmp_log(LOG_WARNING,"context passed during registration does not "
                  "equal the reginfo contextName! ('%s' != '%s')\n",
                  context, reginfo->contextName);
-        netsnmp_assert("register context" == "reginfo->contextName");
+        netsnmp_assert(!"register context == reginfo->contextName"); /* always false */
     }
 
     /*  Create the new subtree node being registered.  */
@@ -1216,7 +1220,7 @@ in_a_view(oid *name, size_t *namelen, netsnmp_pdu *pdu, int type)
     /*
      * check for v1 and counter64s, since snmpv1 doesn't support it 
      */
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (pdu->version == SNMP_VERSION_1 && type == ASN_COUNTER64) {
         return VACM_NOTINVIEW;
     }
@@ -1233,10 +1237,10 @@ in_a_view(oid *name, size_t *namelen, netsnmp_pdu *pdu, int type)
     view_parms.check_subtree = 0;
 
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
 #endif
     case SNMP_VERSION_3:
@@ -1268,10 +1272,10 @@ check_access(netsnmp_pdu *pdu)
     }
 
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
 #endif
     case SNMP_VERSION_3:
@@ -1305,10 +1309,10 @@ netsnmp_acm_check_subtree(netsnmp_pdu *pdu, oid *name, size_t namelen)
     }
 
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
 #endif
     case SNMP_VERSION_3:
@@ -1458,7 +1462,7 @@ netsnmp_subtree_find_prev(oid *name, size_t len, netsnmp_subtree *subtree,
     lookup_cache *lookup_cache = NULL;
     netsnmp_subtree *myptr = NULL, *previous = NULL;
     int cmp = 1;
-    int ll_off = 0;
+    size_t ll_off = 0;
 
     if (subtree) {
         myptr = subtree;
