@@ -266,6 +266,8 @@ extend_load_cache(netsnmp_cache *cache, void *magic)
             extension->lines = calloc( sizeof(char *), extension->numlines );
             memcpy( extension->lines, line_buf,
                                        sizeof(char *) * extension->numlines );
+        } else {
+            extension->lines = &extension->output;
         }
     }
     extension->result = ret;
@@ -284,10 +286,10 @@ extend_free_cache(netsnmp_cache *cache, void *magic)
         SNMP_FREE(extension->output);
         extension->output = NULL;
     }
-    if (extension->lines) {
+    if ( extension->numlines > 1 ) {
         SNMP_FREE(extension->lines);
-        extension->lines  = NULL;
     }
+    extension->lines  = NULL;
     extension->out_len  = 0;
     extension->numlines = 0;
 }
@@ -1017,7 +1019,7 @@ handle_nsExtendOutput1Table(netsnmp_mib_handler          *handler,
                  * find the length of the first one.
                  * Otherwise find the length of the whole string.
                  */
-                if (extension->lines) {
+                if (extension->numlines > 1) {
                     len = (extension->lines[1])-(extension->output) -1;
                 } else if (extension->output) {
                     len = strlen(extension->output);
@@ -1032,7 +1034,7 @@ handle_nsExtendOutput1Table(netsnmp_mib_handler          *handler,
                 snmp_set_var_typed_value(
                      request->requestvb, ASN_OCTET_STR,
                      extension->output,
-                    (extension->output)?strlen(extension->output):0);
+                    (extension->output)?extension->out_len:0);
                 break;
             case COLUMN_EXTOUT1_NUMLINES:
                 snmp_set_var_typed_value(
@@ -1081,6 +1083,7 @@ _extend_find_entry( netsnmp_request_info       *request,
     int oid_len;
     int i;
     char *token;
+    int   token_len;
 
     if (!request || !table_info || !table_info->indexes
                  || !table_info->indexes->next_variable) {
@@ -1139,8 +1142,9 @@ _extend_find_entry( netsnmp_request_info       *request,
                 }
             }
         } else {
-            token    =  table_info->indexes->val.string,
-            line_idx = *table_info->indexes->next_variable->val.integer;
+            token     =  table_info->indexes->val.string;
+            token_len =  table_info->indexes->val_len;
+            line_idx  = *table_info->indexes->next_variable->val.integer;
             DEBUGMSGTL(( "nsExtendTable:output2", "GETNEXT: %s / %d\n ",
                           token, line_idx ));
             /*
@@ -1148,9 +1152,9 @@ _extend_find_entry( netsnmp_request_info       *request,
              * than the requested token...
              */
             for (eptr = ereg->ehead; eptr; eptr = eptr->next ) {
-                if ( strlen(eptr->token) > strlen( token ))
+                if ( strlen(eptr->token) > token_len )
                     break;
-                if ( strlen(eptr->token) == strlen( token ) &&
+                if ( strlen(eptr->token) == token_len &&
                      strcmp(eptr->token, token) >= 0 )
                     break;
             }
@@ -1196,6 +1200,13 @@ _extend_find_entry( netsnmp_request_info       *request,
                     line_idx++;
                 }
             }
+            else {
+                /*
+                 * If this is not the same entry that was requested,
+                 * then we should return the first line.
+                 */
+                line_idx = 1;
+            }
         }
         if (eptr) {
             DEBUGMSGTL(( "nsExtendTable:output2", "GETNEXT -> %s / %d\n ",
@@ -1205,6 +1216,7 @@ _extend_find_entry( netsnmp_request_info       *request,
              * now we've found the appropriate entry (and line),
              * we need to update the varbind OID ...
              */
+            memset(oid_buf, 0, sizeof(oid_buf));
             oid_len = ereg->oid_len;
             memcpy( oid_buf, ereg->root_oid, oid_len*sizeof(oid));
             oid_buf[ oid_len++ ] = 4;    /* nsExtendOutput2Table */
@@ -1223,7 +1235,8 @@ _extend_find_entry( netsnmp_request_info       *request,
              */
             snmp_set_var_value( table_info->indexes,
                                 eptr->token, strlen(eptr->token));
-            *table_info->indexes->next_variable->val.integer = line_idx;
+            snmp_set_var_value( table_info->indexes->next_variable,
+                                &line_idx, sizeof(line_idx));
         }
         return eptr;  /* Finally, signal success */
     }
@@ -1278,10 +1291,7 @@ handle_nsExtendOutput2Table(netsnmp_mib_handler          *handler,
                  * Determine which line we've been asked for....
                  */
                 line_idx = *table_info->indexes->next_variable->val.integer;
-                if (extension->lines)
-                    cp  = extension->lines[line_idx-1];
-                else
-                    cp  = extension->output;
+                cp  = extension->lines[line_idx-1];
 
                 /* 
                  * ... and how long it is.
@@ -1350,7 +1360,7 @@ var_extensible_old(struct variable * vp,
             return ((u_char *) (&long_ret));
         case ERRORMSG:         /* first line of text returned from the process */
             netsnmp_cache_check_and_reload( exten->exec_entry->cache );
-            if (exten->exec_entry->lines) {
+            if (exten->exec_entry->numlines > 1) {
                 *var_len = (exten->exec_entry->lines[1])-
                            (exten->exec_entry->output) -1;
             } else if (exten->exec_entry->output) {
