@@ -226,9 +226,15 @@ agentx_realloc_build_oid(u_char ** buf, size_t * buf_len, size_t * out_len,
     DEBUGMSGOID(("dumpv_send", name, name_len));
     DEBUGMSG(("dumpv_send", "\n"));
 
-    if (name_len == 2 && (name[0] == 0 && name[1] == 0)) {
-        name_len = 0;           /* Null OID */
-    }
+    /*
+     * Updated clarification from the AgentX mailing list.
+     * The "null Object Identifier" mentioned in RFC 2471,
+     * section 5.1 is a special placeholder value, and
+     * should only be used when explicitly mentioned in
+     * this RFC.  In particular, it does *not* mean {0, 0}
+     */
+    if (name_len == 0)
+        inclusive = 0;
 
     /*
      * 'Compact' internet OIDs 
@@ -331,7 +337,7 @@ agentx_realloc_build_string(u_char ** buf, size_t * buf_len,
     return 1;
 }
 
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
 int
 agentx_realloc_build_double(u_char ** buf, size_t * buf_len,
                             size_t * out_len, int allow_realloc,
@@ -393,7 +399,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
 {
     DEBUGDUMPHEADER("send", "VarBind");
     DEBUGDUMPHEADER("send", "type");
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     if ((vp->type == ASN_OPAQUE_FLOAT) || (vp->type == ASN_OPAQUE_DOUBLE)
         || (vp->type == ASN_OPAQUE_I64) || (vp->type == ASN_OPAQUE_U64)
         || (vp->type == ASN_OPAQUE_COUNTER64)) {
@@ -462,7 +468,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
         }
         break;
 
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     case ASN_OPAQUE_FLOAT:
         DEBUGDUMPHEADER("send", "Build Opaque Float");
         DEBUGPRINTINDENT("dumpv_send");
@@ -1278,7 +1284,7 @@ agentx_parse_opaque(u_char * data, size_t * length, int *type,
 
     buf = opaque_buf;
 
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     if ((buf[0] != ASN_OPAQUE_TAG1) || (*opaque_len <= 3))
         return cp;              /* Unrecognised opaque type */
 
@@ -1333,6 +1339,7 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
     u_int           int_val;
     int            int_offset;
     u_int          *int_ptr = (u_int *) data_buf;
+    struct counter64 tmp64;
 
     DEBUGDUMPHEADER("recv", "VarBind:");
     DEBUGDUMPHEADER("recv", "Type");
@@ -1384,38 +1391,17 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
         break;
 
     case ASN_COUNTER64:
-        /*
-         * Set up offset to be 2 for 64-bit, 1 for 32-bit.
-         * Use this value in formulas to correctly put integer values
-         * extracted from buffer into correct place in byte buffer.
-         */
-        int_offset = sizeof(long) == 8 ? 2 : 1;
-	if (network_byte_order) {
-            /*
-             * For 64-bit, clear integers 2 & 3, then place values in 0 & 1.
-             * For 32-bit, clear integers 0 & 1, then overwrite with values.
-             * Could also put a conditional in here to skip clearing 0 & 1.
-             */
-	    int_ptr[(2 * int_offset) - 2] = 0;
-	    int_ptr[(2 * int_offset) - 1] = 0;
-	    int_ptr[0] = agentx_parse_int(bufp, network_byte_order);
-	    int_ptr[1] = agentx_parse_int(bufp + 4, network_byte_order);
-	} else {
-            /*
-             * For 64-bit, clear integers 0 & 1, then place values in 2 & 3.
-             * For 32-bit, clear integers 0 & 1, then overwrite with values.
-             * Could also put a conditional in here to skip clearing 0 & 1.
-             */
-	    int_ptr[0] = 0;
-	    int_ptr[1] = 0;
-	    int_ptr[(2 * int_offset) - 2] = agentx_parse_int(bufp + 4,
-                                                            network_byte_order);
-	    int_ptr[(2 * int_offset) - 1] = agentx_parse_int(bufp,
-                                                            network_byte_order);
-	}
+        memset(&tmp64, 0, sizeof(tmp64));
+        if (network_byte_order) {
+            tmp64.high = agentx_parse_int(bufp,   network_byte_order);
+            tmp64.low  = agentx_parse_int(bufp+4, network_byte_order);
+        } else {
+            tmp64.high = agentx_parse_int(bufp+4, network_byte_order);
+            tmp64.low  = agentx_parse_int(bufp,   network_byte_order);
+        }
 
-        /* return data_len 2*8 if 64-bit, 2*4 if 32-bit */
-	*data_len = 2 * sizeof(long);
+        memcpy(data_buf, &tmp64, sizeof(tmp64));
+        *data_len = sizeof(tmp64);
 	bufp += 2 * sizeof(long);
 	*length -= 8;
         break;
