@@ -36,10 +36,6 @@
 #include <netinet/in.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/library/snmp_assert.h>
@@ -56,6 +52,8 @@
 
 #include "subagent.h"
 
+#ifdef USING_AGENTX_SUBAGENT_MODULE
+
 static SNMPCallback subagent_register_ping_alarm;
 static SNMPAlarmCallback agentx_reopen_session;
 void            agentx_register_callbacks(netsnmp_session * s);
@@ -67,6 +65,9 @@ int             handle_subagent_set_response(int op,
                                              netsnmp_session * session,
                                              int reqid, netsnmp_pdu *pdu,
                                              void *magic);
+void            subagent_startup_callback(unsigned int clientreg,
+                                          void *clientarg);
+int             subagent_open_master_session(void);
 
 typedef struct _net_snmpsubagent_magic_s {
     int             original_command;
@@ -89,35 +90,79 @@ static struct agent_netsnmp_set_info *Sets = NULL;
 
 netsnmp_session *agentx_callback_sess = NULL;
 extern int      callback_master_num;
+extern netsnmp_session *main_session;   /* from snmp_agent.c */
 
-void
-init_subagent(void)
+int
+subagent_startup(int majorID, int minorID,
+                             void *serverarg, void *clientarg)
 {
-#ifndef SNMP_TRANSPORT_CALLBACK_DOMAIN
+    DEBUGMSGTL(("agentx/subagent", "connecting to master...\n"));
+    /*
+     * if a valid ping interval has been defined, call agentx_reopen_session
+     * to try to connect to master or setup a ping alarm if it couldn't
+     * succeed. if no ping interval was set up, just try to connect once.
+     */
+    if (netsnmp_ds_get_int(NETSNMP_DS_APPLICATION_ID,
+                           NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL) > 0)
+        agentx_reopen_session(0, NULL);
+    else {
+        subagent_open_master_session();
+    }
+    return 0;
+}
+
+/**
+ * init subagent callback (local) session and connect to master agent
+ *
+ * @returns 0 for success, !0 otherwise
+ */
+int
+subagent_init(void)
+{
+    static int init = 0;
+    int rc = 0;
+
+    DEBUGMSGTL(("agentx/subagent", "initializing....\n"));
+
+    if (++init != 1)
+        return 0;
+
+    netsnmp_assert(netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
+                                          NETSNMP_DS_AGENT_ROLE) == SUB_AGENT);
+
+#ifndef NETSNMP_TRANSPORT_CALLBACK_DOMAIN
     snmp_log(LOG_WARNING,"AgentX subagent has been disabled because "
                "the callback transport is not available.\n");
-    return;
-#else
-    agentx_register_config_handler("agentxsocket",
-                                  agentx_parse_agentx_socket, NULL,
-                                  "AgentX bind address");
+    return -1;
+#endif /* NETSNMP_TRANSPORT_CALLBACK_DOMAIN */
+
+    /*
+     * open (local) callback session
+     */
     if (agentx_callback_sess == NULL) {
         agentx_callback_sess = netsnmp_callback_open(callback_master_num,
                                                      handle_subagent_response,
                                                      NULL, NULL);
-        DEBUGMSGTL(("agentx/subagent", "init_subagent sess %08x\n",
+        DEBUGMSGTL(("agentx/subagent", "subagent_init sess %08x\n",
                     agentx_callback_sess));
     }
-#endif /* SNMP_TRANSPORT_CALLBACK_DOMAIN */
+    if (NULL == agentx_callback_sess)
+        return -1;
+
+    snmp_register_callback(SNMP_CALLBACK_LIBRARY,
+                           SNMP_CALLBACK_POST_READ_CONFIG,
+                           subagent_startup, NULL);
+
+    DEBUGMSGTL(("agentx/subagent", "initializing....  DONE\n"));
+
+    return rc;
 }
 
-#ifdef USING_AGENTX_SUBAGENT_MODULE
 void
 netsnmp_enable_subagent(void) {
     netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE,
                            SUB_AGENT);
 }
-#endif /* USING_AGENTX_SUBAGENT_MODULE */
 
 struct agent_netsnmp_set_info *
 save_set_vars(netsnmp_session * ss, netsnmp_pdu *pdu)
@@ -191,8 +236,6 @@ free_set_vars(netsnmp_session * ss, netsnmp_pdu *pdu)
     }
 }
 
-extern netsnmp_session *main_session;   /* from snmp_agent.c */
-
 int
 handle_agentx_packet(int operation, netsnmp_session * session, int reqid,
                      netsnmp_pdu *pdu, void *magic)
@@ -451,7 +494,7 @@ handle_subagent_response(int op, netsnmp_session * session, int reqid,
     }
 
     if (smagic->original_command == AGENTX_MSG_GETNEXT) {
-        DEBUGMSGTL(("agentx/subgaent",
+        DEBUGMSGTL(("agentx/subagent",
                     "do getNext scope processing %p %p\n", smagic->ovars,
                     pdu->variables));
         for (u = smagic->ovars, v = pdu->variables; u != NULL && v != NULL;
@@ -644,9 +687,6 @@ agentx_register_callbacks(netsnmp_session * s)
 {
     DEBUGMSGTL(("agentx/subagent",
                 "registering callbacks for session %p\n", s));
-    snmp_register_callback(SNMP_CALLBACK_LIBRARY,
-                           SNMP_CALLBACK_POST_READ_CONFIG,
-                           subagent_register_ping_alarm, s);
     snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_SHUTDOWN,
                            subagent_shutdown, s);
     snmp_register_callback(SNMP_CALLBACK_APPLICATION,
@@ -674,9 +714,6 @@ agentx_unregister_callbacks(netsnmp_session * ss)
 {
     DEBUGMSGTL(("agentx/subagent",
                 "unregistering callbacks for session %p\n", ss));
-    snmp_unregister_callback(SNMP_CALLBACK_LIBRARY,
-                             SNMP_CALLBACK_POST_READ_CONFIG,
-                             subagent_register_ping_alarm, ss, 1);
     snmp_unregister_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_SHUTDOWN,
                              subagent_shutdown, ss, 1);
     snmp_unregister_callback(SNMP_CALLBACK_APPLICATION,
@@ -702,6 +739,7 @@ agentx_unregister_callbacks(netsnmp_session * ss)
 int
 subagent_open_master_session(void)
 {
+    netsnmp_transport *t;
     netsnmp_session sess;
 
     DEBUGMSGTL(("agentx/subagent", "opening session...\n"));
@@ -717,47 +755,53 @@ subagent_open_master_session(void)
     sess.retries = SNMP_DEFAULT_RETRIES;
     sess.timeout = SNMP_DEFAULT_TIMEOUT;
     sess.flags |= SNMP_FLAGS_STREAM_SOCKET;
-    if (netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_X_SOCKET)) {
-        sess.peername = strdup(
-            netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_X_SOCKET));
-    } else {
-        sess.peername = strdup(AGENTX_SOCKET);
-    }
-
-    sess.local_port = 0;        /* client */
-    sess.remote_port = AGENTX_PORT;     /* default port */
     sess.callback = handle_agentx_packet;
     sess.authenticator = NULL;
-    main_session = snmp_open_ex(&sess, NULL, agentx_parse, NULL, NULL,
-                                agentx_realloc_build, agentx_check_packet);
 
-    if (main_session == NULL) {
+    t = netsnmp_transport_open_client(
+            "agentx", netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID,
+                                            NETSNMP_DS_AGENT_X_SOCKET));
+    if (t == NULL) {
         /*
          * Diagnose snmp_open errors with the input
          * netsnmp_session pointer.  
          */
-        if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_NO_CONNECTION_WARNINGS)) {
+        if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
+                                    NETSNMP_DS_AGENT_NO_CONNECTION_WARNINGS)) {
             char buf[1024];
-            if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_NO_ROOT_ACCESS)) {
-                snprintf(buf, sizeof(buf), "Warning: "
-                         "Failed to connect to the agentx master agent (%s)",
-                         sess.peername);
+            const char *socket =
+                netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID,
+                                      NETSNMP_DS_AGENT_X_SOCKET);
+            snprintf(buf, sizeof(buf), "Warning: "
+                     "Failed to connect to the agentx master agent (%s)",
+                     socket ? socket : "[NIL]");
+            if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
+                                        NETSNMP_DS_AGENT_NO_ROOT_ACCESS)) {
                 netsnmp_sess_log_error(LOG_WARNING, buf, &sess);
             } else {
-                snprintf(buf, sizeof(buf), "Error: "
-                         "Failed to connect to the agentx master agent (%s)",
-                         sess.peername);
                 snmp_sess_perror(buf, &sess);
             }
         }
-        if (sess.peername)
-            /* was memduped above and is no longer needed */
-            free(sess.peername);
         return -1;
     }
-    if (sess.peername)
-        /* was memduped above and is no longer needed */
-        free(sess.peername);
+
+    main_session =
+        snmp_add_full(&sess, t, NULL, agentx_parse, NULL, NULL,
+                      agentx_realloc_build, agentx_check_packet, NULL);
+
+    if (main_session == NULL) {
+        if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
+                                    NETSNMP_DS_AGENT_NO_CONNECTION_WARNINGS)) {
+            char buf[1024];
+            snprintf(buf, sizeof(buf), "Error: "
+                     "Failed to create the agentx master agent session (%s)",
+                     netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID,
+                                           NETSNMP_DS_AGENT_X_SOCKET));
+            snmp_sess_perror(buf, &sess);
+        }
+        netsnmp_transport_free(t);
+        return -1;
+    }
 
     /*
      * I don't know why 1 is success instead of the usual 0 = noerr, 
@@ -793,51 +837,6 @@ subagent_open_master_session(void)
     return 0;
 }
 
-/*
- * returns non-zero on error 
- */
-int
-subagent_pre_init(void)
-{
-    DEBUGMSGTL(("agentx/subagent", "initializing....\n"));
-
-    /*
-     * set up callbacks to initiate master agent pings for this session 
-     */
-    netsnmp_ds_register_config(ASN_INTEGER,
-                               netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID,
-                                                     NETSNMP_DS_LIB_APPTYPE),
-                               "agentxPingInterval",
-                               NETSNMP_DS_APPLICATION_ID,
-                               NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL);
-
-    
-    /* ping and/or reconnect by default every 15 seconds */
-    netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
-                       NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL, 15);
-
-
-    if (netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE) != SUB_AGENT) {
-        return 0;
-    }
-
-    /*
-     * if a valid ping interval has been defined, call agentx_reopen_session
-     * to try to connect to master or setup a ping alarm if it couldn't
-     * succeed
-     */
-    if (netsnmp_ds_get_int(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL) > 0)
-        agentx_reopen_session(0, NULL);
-    else                        /* if no ping interval was set up, just try to connect once */
-        subagent_open_master_session();
-    if (!main_session)
-        return -1;
-
-    DEBUGMSGTL(("agentx/subagent", "initializing....  DONE\n"));
-
-    return 0;
-}
-
 
 /*
  * Alarm callback function to open a session to the master agent.  If a
@@ -957,12 +956,22 @@ agentx_check_session(unsigned int clientreg, void *clientarg)
         snmp_alarm_unregister(clientreg);       /* delete ping alarm timer */
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_INDEX_STOP, (void *) ss);
-        snmp_close(main_session);
         register_mib_detach();
+        if (main_session != NULL) {
+            remove_trap_session(ss);
+        snmp_close(main_session);
         main_session = NULL;
         agentx_reopen_session(0, NULL);
+        }
+        else {
+            snmp_close(main_session);
+            main_session = NULL;
+        }
     } else {
         DEBUGMSGTL(("agentx/subagent", "session %p responded to ping\n",
                     ss));
     }
 }
+
+
+#endif /* USING_AGENTX_SUBAGENT_MODULE */
