@@ -6,6 +6,7 @@
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/agent/agent_callbacks.h>
 #include "disman/event/mteObjects.h"
 #include "disman/event/mteEvent.h"
 #include "disman/event/mteEventConf.h"
@@ -51,6 +52,9 @@ init_mteEventConf(void)
      */
     snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_STORE_DATA,
                            store_mteETable, NULL);
+    snmp_register_callback(SNMP_CALLBACK_APPLICATION,
+                           SNMPD_CALLBACK_PRE_UPDATE_CONFIG,
+                           clear_mteETable, NULL);
 }
 
 
@@ -87,6 +91,7 @@ _find_mteEvent_entry( char *owner, char *ename )
     if (!row)
         return NULL;
 
+    /* return (struct mteEvent *)netsnmp_tdata_row_entry( row ); */
     return (struct mteEvent *)row->data;
 }
 
@@ -131,7 +136,9 @@ parse_notificationEvent( const char *token, char *line )
     int    wild = 1;
     int    idx  = 0;
     char  *cp;
+#ifndef NETSNMP_DISABLE_MIB_LOADING
     struct tree         *tp;
+#endif
     struct varbind_list *var;
 
     DEBUGMSGTL(("disman:event:conf", "Parsing notificationEvent config\n"));
@@ -171,6 +178,10 @@ parse_notificationEvent( const char *token, char *line )
      *  ... and the relevant object/instances.
      */
     if ( cp && *cp=='-' && *(cp+1)=='m' ) {
+#ifdef NETSNMP_DISABLE_MIB_LOADING
+        config_perror("Can't use -m if MIB loading is disabled");
+        return;
+#else
         /*
          * Use the MIB definition to add the standard
          *   notification payload to the mteObjectsTable.
@@ -187,6 +198,7 @@ parse_notificationEvent( const char *token, char *line )
                                          var->vblabel, wild );
             idx    = object->mteOIndex;
         }
+#endif
     }
     while (cp) {
         if ( *cp == '-' ) {
@@ -546,3 +558,26 @@ store_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
     DEBUGMSGTL(("disman:event:conf", "  done.\n"));
     return SNMPERR_SUCCESS;
 }
+
+int
+clear_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
+{
+    netsnmp_tdata_row    *row;
+    netsnmp_variable_list owner_var;
+
+    /*
+     * We're only interested in entries set up via the config files
+     */
+    memset( &owner_var, 0, sizeof(netsnmp_variable_list));
+    snmp_set_var_typed_value( &owner_var,  ASN_OCTET_STR,
+                             "snmpd.conf", strlen("snmpd.conf"));
+    while (( row = netsnmp_tdata_row_next_byidx( event_table_data,
+                                                &owner_var ))) {
+        /*
+         * XXX - check for owner of "snmpd.conf"
+         *       and break at the end of these
+         */
+        netsnmp_tdata_remove_and_delete_row( event_table_data, row );
+    }
+    return SNMPERR_SUCCESS;
+}
