@@ -392,7 +392,7 @@ var_hrswinst(struct variable * vp,
             else
                 long_return = 0;        /* predates this agent */
         } else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
             return NULL;
 #else
             long_return = 363136200;
@@ -441,7 +441,19 @@ var_hrswinst(struct variable * vp,
                 free(catg);
             }
 #else
+# ifdef HAVE_LIBRPM
+            char *rpm_groups;
+            if ( headerGetEntry(swi->swi_h, RPMTAG_GROUP, NULL, (void **) &rpm_groups, NULL) ) {
+                if ( strstr(rpm_groups, "System Environment") != NULL )
+                    long_return = 2;	/* operatingSystem */
+                else
+                    long_return = 4;	/* applcation */
+            } else {
+                long_return = 1;    /* unknown */
+            }
+# else
             long_return = 1;    /* unknown */
+# endif
 #endif
             ret = (u_char *) & long_return;
         }
@@ -450,9 +462,7 @@ var_hrswinst(struct variable * vp,
         {
 #ifdef HAVE_LIBRPM
             int_32         *rpm_data;
-            headerGetEntry(swi->swi_h, RPMTAG_INSTALLTIME, NULL,
-                           (void **) &rpm_data, NULL);
-            if (rpm_data != NULL) {
+            if ( headerGetEntry(swi->swi_h, RPMTAG_INSTALLTIME, NULL, (void **) &rpm_data, NULL) ) {
                 time_t          installTime = *rpm_data;
                 ret = date_n_time(&installTime, var_len);
             } else {
@@ -466,7 +476,7 @@ var_hrswinst(struct variable * vp,
                 stat(string, &stat_buf);
                 ret = date_n_time(&stat_buf.st_mtime, var_len);
             } else {
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
                 return NULL;
 #endif
                 sprintf(string, "back in the mists of time");
