@@ -83,8 +83,12 @@
 #include "hr_swrun.h"
 #include <net-snmp/agent/auto_nlist.h>
 #include "kernel.h"
-#if solaris2
+#ifdef solaris2
 #include "kernel_sunos5.h"
+#endif
+#if defined(aix4) || defined(aix5)
+#include <procinfo.h>
+#include <sys/types.h>
 #endif
 
         /*********************
@@ -117,6 +121,8 @@ struct pst_dynamic pst_dyn;
 struct kinfo_proc *proc_table;
 #elif defined(solaris2)
 int            *proc_table;
+#elif defined(aix4) || defined(aix5)
+struct procsinfo *proc_table;
 #else
 struct proc    *proc_table;
 #endif
@@ -523,9 +529,9 @@ var_hrswrun(struct variable * vp,
             ret =  read(procfd, proc_buf, sizeof(*proc_buf));
             close(procfd);
             if (ret != sizeof(*proc_buf))
-                return NULL;
+                proc_buf = NULL;
         } else
-            return NULL;
+            proc_buf =  NULL;
 #else
         if (kd == NULL)
             return NULL;
@@ -539,7 +545,7 @@ var_hrswrun(struct variable * vp,
 
     switch (vp->magic) {
     case HRSWRUN_OSINDEX:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #else
         /* 
@@ -584,6 +590,12 @@ var_hrswrun(struct variable * vp,
         strncpy(string, proc_buf->p_user.u_comm, sizeof(string));
         string[ sizeof(string)-1 ] = 0;
 #endif
+#elif defined(aix4) || defined(aix5)
+        strncpy(string, proc_table[LowProcIndex].pi_comm, sizeof(string));
+        string[ sizeof(string)-1 ] = 0;
+        cp = strchr(string, ' ');
+        if (cp != NULL)
+            *cp = '\0';
 #elif HAVE_KVM_GETPROCS
     #if defined(freebsd5) && __FreeBSD_version >= 500014
         strcpy(string, proc_table[LowProcIndex].ki_comm);
@@ -646,7 +658,7 @@ var_hrswrun(struct variable * vp,
         if (cp > string && strcasecmp(cp, ".exe") == 0)
             *cp = '\0';
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         sprintf(string, "process name");
@@ -701,6 +713,12 @@ var_hrswrun(struct variable * vp,
             *cp1++ = *cp++;
         *cp1 = 0;
 #endif
+#elif defined(aix4) || defined(aix5)
+        strncpy(string, proc_table[LowProcIndex].pi_comm, sizeof(string));
+        string[ sizeof(string)-1 ] = 0;
+        cp = strchr(string, ' ');
+        if (cp != NULL)
+            *cp = '\0';
 #elif HAVE_KVM_GETPROCS
     #if defined(freebsd5) && __FreeBSD_version >= 500014
         strcpy(string, proc_table[LowProcIndex].ki_comm);
@@ -760,7 +778,7 @@ var_hrswrun(struct variable * vp,
         } else
             strcpy(string, "* unknown");
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         sprintf(string, "/bin/wombat");
@@ -804,6 +822,13 @@ var_hrswrun(struct variable * vp,
             cp++;
         strcpy(string, cp);
 #endif
+#elif defined(aix4) || defined(aix5)
+        cp = strchr(proc_table[LowProcIndex].pi_comm, ' ');
+        if (cp != NULL) {
+            cp++;
+            sprintf(string, "%s", cp);
+        } else
+            string[0] = '\0';
 #elif HAVE_KVM_GETPROCS
         string[0] = 0;
         argv = kvm_getargv(kd, proc_table + LowProcIndex, sizeof(string));
@@ -864,7 +889,7 @@ var_hrswrun(struct variable * vp,
 #elif defined(cygwin)
         string[0] = 0;
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         sprintf(string, "-h -q -v");
@@ -881,6 +906,11 @@ var_hrswrun(struct variable * vp,
             long_return = 2;    /* operatingSystem */
         else
             long_return = 4;    /* application */
+#elif defined(aix4) || defined(aix5)
+		if (proc_table[LowProcIndex].pi_flags & SKPROC) {
+			long_return = 2;	/* kernel process */
+		} else
+			long_return = 4;	/* application */
 #elif HAVE_KVM_GETPROCS
     #if defined(freebsd5) && __FreeBSD_version >= 500014
 	if (proc_table[LowProcIndex].ki_flag & P_SYSTEM) {
@@ -943,6 +973,8 @@ var_hrswrun(struct variable * vp,
 #else
         switch (proc_buf->p_stat) {
 #endif
+#elif defined(aix4) || defined(aix5)
+        switch (proc_table[LowProcIndex].pi_state) {
 #else
         switch (proc_table[LowProcIndex].p_stat) {
 #endif
@@ -1029,7 +1061,7 @@ var_hrswrun(struct variable * vp,
         long_return = proc_buf->p_utime * 100 + proc_buf->p_stime * 100;
 #endif
 #elif HAVE_KVM_GETPROCS
-    #if defined(freebsd5) && __FreeBSD_version >= 500014
+    #if defined(NOT_DEFINED) && defined(freebsd5) && __FreeBSD_version >= 500014
         /* XXX: Accessing ki_paddr causes sig10 ...
         long_return = proc_table[LowProcIndex].ki_paddr->p_uticks +
             proc_table[LowProcIndex].ki_paddr->p_sticks +
@@ -1088,11 +1120,11 @@ var_hrswrun(struct variable * vp,
                 long_return = 0;
             }
         }
-#elif defined(aix4)
-        long_return = proc_table[LowProcIndex].p_ru.ru_utime.tv_sec * 100 +
-            proc_table[LowProcIndex].p_ru.ru_utime.tv_usec / 10000 +
-            proc_table[LowProcIndex].p_ru.ru_stime.tv_sec * 100 +
-            proc_table[LowProcIndex].p_ru.ru_stime.tv_usec / 10000;
+#elif defined(aix4) || defined(aix5)
+        long_return = proc_table[LowProcIndex].pi_ru.ru_utime.tv_sec * 100 +
+            proc_table[LowProcIndex].pi_ru.ru_utime.tv_usec / 10000000 + /* nanoseconds */
+            proc_table[LowProcIndex].pi_ru.ru_stime.tv_sec * 100 +
+            proc_table[LowProcIndex].pi_ru.ru_stime.tv_usec / 10000000; /* nanoseconds */
 #else
         long_return = proc_table[LowProcIndex].p_utime.tv_sec * 100 +
             proc_table[LowProcIndex].p_utime.tv_usec / 10000 +
@@ -1111,8 +1143,10 @@ var_hrswrun(struct variable * vp,
 #else
         long_return = proc_buf->p_swrss;
 #endif
+#elif defined(aix4) || defined(aix5)
+        long_return = proc_table[LowProcIndex].pi_size * getpagesize() / 1024;
 #elif HAVE_KVM_GETPROCS && !defined(darwin8)
-  #if defined(freebsd5) && __FreeBSD_version >= 500014
+  #if defined(NOT_DEFINED) && defined(freebsd5) && __FreeBSD_version >= 500014
 	    /* XXX
 	    long_return = proc_table[LowProcIndex].ki_vmspace->vm_tsize +
 			  proc_table[LowProcIndex].ki_vmspace->vm_ssize +
@@ -1172,7 +1206,7 @@ var_hrswrun(struct variable * vp,
             }
         }
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 16 * 1024;        /* XXX - 16M! */
@@ -1348,8 +1382,41 @@ Init_HR_SWRun(void)
         while ((dp = readdir(f)) != NULL && current_proc_entry < nproc)
             if (dp->d_name[0] != '.')
                 proc_table[current_proc_entry++] = atoi(dp->d_name);
+        /*
+         * if we are in a Solaris zone, nproc > current_proc_entry !
+         * but we only want the processes from the local zone
+         */
+        if (current_proc_entry != nproc)
+            nproc = current_proc_entry;
         closedir(f);
     }
+#elif defined(aix4) || defined(aix5)
+    {
+		pid_t proc_index = 0;
+		int avail = 1024;
+		if (proc_table) {
+			free(proc_table);
+		}
+		nproc = 0;
+		proc_table = malloc(sizeof(proc_table[0]) * avail);
+		for (;;) {
+			int got;
+			if (!proc_table) {
+				nproc = 0;
+				snmp_log_perror("Init_HR_SWRun-malloc");
+				return;
+			}
+			got = getprocs(proc_table + nproc, sizeof(proc_table[0]),
+						   0, sizeof(struct fdsinfo),
+						   &proc_index, avail - nproc);
+			nproc += got;
+			if (nproc < avail) {
+				break;
+			}
+			avail += 1024;
+			proc_table = realloc(proc_table, avail * sizeof(proc_table[0]));
+		}
+    }
 #elif HAVE_KVM_GETPROCS
     {
         if (kd == NULL) {
@@ -1388,7 +1455,7 @@ Init_HR_SWRun(void)
             snmp_log_perror("Init_HR_SWRun-auto_nlist PROC");
             return;
         }
-        if (klookup(proc_table_base, (char *) proc_table, bytes) == 0) {
+        if (NETSNMP_KLOOKUP(proc_table_base, (char *) proc_table, bytes) == 0) {
             nproc = 0;
             snmp_log_perror("Init_HR_SWRun-klookup");
             return;
@@ -1414,6 +1481,11 @@ Get_Next_HR_SWRun(void)
         if (proc_table[current_proc_entry].kp_proc.p_stat != 0)
             return proc_table[current_proc_entry++].kp_proc.p_pid;
     #endif
+#elif defined(aix4) || defined(aix5)
+        if (proc_table[current_proc_entry].pi_state != 0)
+            return proc_table[current_proc_entry++].pi_pid;
+        else
+            ++current_proc_entry;
 #else
         if (proc_table[current_proc_entry].p_stat != 0)
             return proc_table[current_proc_entry++].p_pid;
@@ -1444,7 +1516,10 @@ count_processes(void)
 #if defined(hpux10) || defined(hpux11) || HAVE_KVM_GETPROCS || defined(solaris2)
     total = nproc;
 #else
-#if !defined(linux) && !defined(cygwin) && !defined(dynix)
+#if defined(aix4) || defined(aix5)
+    for (i = 0; i < nproc; ++i) {
+        if (proc_table[i].pi_state != 0)
+#elif !defined(linux) && !defined(cygwin) && !defined(dynix)
     for (i = 0; i < nproc; ++i) {
         if (proc_table[i].p_stat != 0)
 #else
