@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface.c,v 1.4.2.1 2006/01/25 16:26:38 dts12 Exp $
+ * $Id: interface.c 15768 2007-01-22 16:18:29Z rstory $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -114,12 +114,14 @@ netsnmp_access_interface_container_init(u_int flags)
     if (NULL == container1)
         return NULL;
 
+    container1->container_name = strdup("interface container");
     if (flags & NETSNMP_ACCESS_INTERFACE_INIT_ADDL_IDX_BY_NAME) {
         netsnmp_container *container2 =
             netsnmp_container_find("access_interface_by_name:access_interface:table_container");
         if (NULL == container2)
             return NULL;
 
+        container2->container_name = strdup("interface name container");
         container2->compare = _access_interface_entry_compare_name;
         
         netsnmp_container_add_index(container1, container2);
@@ -604,7 +606,8 @@ netsnmp_access_interface_entry_copy(netsnmp_interface_entry * lhs,
     lhs->mtu = rhs->mtu;
     lhs->lastchange = rhs->lastchange;
     lhs->discontinuity = rhs->discontinuity;
-    lhs->reasm_max = rhs->reasm_max;
+    lhs->reasm_max_v4 = rhs->reasm_max_v4;
+    lhs->reasm_max_v6 = rhs->reasm_max_v6;
     lhs->admin_status = rhs->admin_status;
     lhs->oper_status = rhs->oper_status;
     lhs->promiscuous = rhs->promiscuous;
