@@ -1,7 +1,7 @@
 /*
  *  Ipaddress MIB architecture support
  *
- * $Id: ipaddress_common.c,v 1.12.2.1 2006/01/25 16:26:38 dts12 Exp $
+ * $Id: ipaddress_common.c 16612 2007-07-16 23:59:44Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -171,6 +171,7 @@ netsnmp_access_ipaddress_entry_create(void)
     if (SNMP_ERR_NOERROR != rc) {
         DEBUGMSGT(("access:ipaddress:create","error %d in arch init\n"));
         netsnmp_access_ipaddress_entry_free(entry);
+        entry = NULL;
     }
 
     return entry;
@@ -238,7 +239,7 @@ netsnmp_access_ipaddress_entry_set(netsnmp_ipaddress_entry * entry)
     }
     else {
         snmp_log(LOG_ERR,"netsnmp_access_ipaddress_entry_set with no mode\n");
-        netsnmp_assert("ipaddress_entry_set" == "unknown mode"); 
+        netsnmp_assert(!"ipaddress_entry_set == unknown mode"); /* always false */
         rc = -1;
     }
     
@@ -358,15 +359,15 @@ netsnmp_ipaddress_prefix_copy(u_char *dst, u_char *src, int addr_len, int pfx_le
 
     memcpy(dst, src, bytes);
 
+    if (bytes < addr_len)
+        memset(&dst[bytes],0x0, addr_len - bytes);
+
     if (bits) {
-        u_char mask = (0xff >> bits);
+        u_char mask = (0xff << (8-bits));
         
         dst[bytes] = (src[bytes] & mask);
     }
 
-    if (bytes < addr_len)
-        memset(&dst[bytes],0x0, addr_len - bytes);
-
     return pfx_len;
 }
 
