@@ -15,6 +15,8 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include "mibII_common.h" /* for NETSNMP_KLOOKUP */
+
 #if HAVE_STRING_H
 #include <string.h>
 #else
@@ -105,9 +107,9 @@
 
 #include <net-snmp/data_access/interface.h>
 
-#if defined(HAVE_SYS_SYSCTL_H) && !defined(CAN_USE_SYSCTL)
-# if defined(RTF_LLINFO) && !defined(irix6)
-#  define CAN_USE_SYSCTL 1
+#if defined(HAVE_SYS_SYSCTL_H) && !defined(NETSNMP_CAN_USE_SYSCTL)
+# if defined(RTF_LLINFO) 
+#  define NETSNMP_CAN_USE_SYSCTL 1
 # endif
 #endif
 
@@ -304,7 +306,7 @@ var_atEntry(struct variable *vp,
 #ifdef ARP_SCAN_FOUR_ARGUMENTS
         long_return = lowIfIndex;
 #else                           /* ARP_SCAN_FOUR_ARGUMENTS */
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 1;        /* XXX */
@@ -314,7 +316,7 @@ var_atEntry(struct variable *vp,
         *var_len = sizeof(LowPhysAddr);
         return (u_char *) LowPhysAddr;
     case IPMEDIANETADDRESS:    /* also ATNETADDRESS */
-        *var_len = sizeof long_return;
+	*var_len = sizeof(uint32_t);
         long_return = LowAddr;
         return (u_char *) & long_return;
     case IPMEDIATYPE:
@@ -339,17 +341,23 @@ AT_Cmp(void *addr, void *ep)
 {
     mib2_ipNetToMediaEntry_t *mp = (mib2_ipNetToMediaEntry_t *) ep;
     int             ret = -1;
+    oid             index;
+
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+    mp->ipNetToMediaIfIndex.o_bytes[mp->ipNetToMediaIfIndex.o_length] = '\0';
+    index = netsnmp_access_interface_index_find(
+                    mp->ipNetToMediaIfIndex.o_bytes);
+#else
+    index = Interface_Index_By_Name(mp->ipNetToMediaIfIndex.o_bytes,
+                                    mp->ipNetToMediaIfIndex.o_length);
+#endif
     DEBUGMSGTL(("mibII/at", "......... AT_Cmp %lx<>%lx %d<>%d (%.5s)\n",
                 mp->ipNetToMediaNetAddress, ((if_ip_t *) addr)->ipAddr,
-                ((if_ip_t *) addr)->ifIdx,
-                Interface_Index_By_Name(mp->ipNetToMediaIfIndex.o_bytes,
-                                        mp->ipNetToMediaIfIndex.o_length),
+                ((if_ip_t *) addr)->ifIdx, index,
                 mp->ipNetToMediaIfIndex.o_bytes));
     if (mp->ipNetToMediaNetAddress != ((if_ip_t *) addr)->ipAddr)
         ret = 1;
-    else if (((if_ip_t *) addr)->ifIdx !=
-             Interface_Index_By_Name(mp->ipNetToMediaIfIndex.o_bytes,
-                                     mp->ipNetToMediaIfIndex.o_length))
+    else if (((if_ip_t *) addr)->ifIdx != index)
         ret = 1;
     else
         ret = 0;
@@ -401,9 +409,15 @@ var_atEntry(struct variable * vp,
             (MIB_IP_NET, &entry, sizeof(mib2_ipNetToMediaEntry_t),
              req_type, &AT_Cmp, &NextAddr) != 0)
             break;
-        current[AT_IFINDEX_OFF] =
-            Interface_Index_By_Name(entry.ipNetToMediaIfIndex.o_bytes,
-                                    entry.ipNetToMediaIfIndex.o_length);
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+        entry.ipNetToMediaIfIndex.o_bytes[entry.ipNetToMediaIfIndex.o_length] = '\0';
+        current[AT_IFINDEX_OFF] = netsnmp_access_interface_index_find(
+           entry.ipNetToMediaIfIndex.o_bytes);
+#else
+        current[AT_IFINDEX_OFF] = 
+           Interface_Index_By_Name(entry.ipNetToMediaIfIndex.o_bytes,
+                                   entry.ipNetToMediaIfIndex.o_length);
+#endif
         if (current[6] == 3) {  /* AT group oid */
             current[AT_IFINDEX_OFF + 1] = 1;
             offset = AT_IFINDEX_OFF + 2;
@@ -446,15 +460,22 @@ var_atEntry(struct variable * vp,
     switch (vp->magic) {
     case IPMEDIAIFINDEX:
         *var_len = sizeof long_return;
-        long_return =
-            Interface_Index_By_Name(Lowentry.ipNetToMediaIfIndex.o_bytes,
-                                    Lowentry.ipNetToMediaIfIndex.o_length);
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+        Lowentry.ipNetToMediaIfIndex.o_bytes[
+            Lowentry.ipNetToMediaIfIndex.o_length] = '\0';
+        long_return = netsnmp_access_interface_index_find(
+            Lowentry.ipNetToMediaIfIndex.o_bytes);
+#else
+        long_return = Interface_Index_By_Name(
+            Lowentry.ipNetToMediaIfIndex.o_bytes,
+            Lowentry.ipNetToMediaIfIndex.o_length);
+#endif
         return (u_char *) & long_return;
     case IPMEDIAPHYSADDRESS:
         *var_len = Lowentry.ipNetToMediaPhysAddress.o_length;
         return Lowentry.ipNetToMediaPhysAddress.o_bytes;
     case IPMEDIANETADDRESS:
-        *var_len = sizeof(Lowentry.ipNetToMediaNetAddress);
+	*var_len = sizeof(uint32_t);
         return (u_char *) &Lowentry.ipNetToMediaNetAddress;
     case IPMEDIATYPE:
         *var_len = sizeof long_return;
@@ -478,7 +499,7 @@ var_atEntry(struct variable * vp,
 #ifndef solaris2
 
 static int      arptab_size, arptab_current;
-#if CAN_USE_SYSCTL
+#if NETSNMP_CAN_USE_SYSCTL
 static char    *lim, *rtnext;
 static char    *at = 0;
 #else
@@ -503,12 +524,12 @@ static struct arptab *at = NULL;
 static int      arptab_curr_max_size = 0;
 
 #endif
-#endif                          /* CAN_USE_SYSCTL */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL */
 
 static void
 ARP_Scan_Init(void)
 {
-#ifndef CAN_USE_SYSCTL
+#ifndef NETSNMP_CAN_USE_SYSCTL
 #ifndef linux
 #ifdef hpux11
 
@@ -549,12 +570,16 @@ ARP_Scan_Init(void)
 #else                           /* hpux11 */
 
     if (!at) {
+#ifdef ARPTAB_SIZE_SYMBOL
         auto_nlist(ARPTAB_SIZE_SYMBOL, (char *) &arptab_size,
                    sizeof arptab_size);
 #ifdef STRUCT_ARPHD_HAS_AT_NEXT
         at = (struct arphd *) malloc(arptab_size * sizeof(struct arphd));
 #else
         at = (struct arptab *) malloc(arptab_size * sizeof(struct arptab));
+#endif
+#else
+        return;
 #endif
     }
 #ifdef STRUCT_ARPHD_HAS_AT_NEXT
@@ -648,7 +673,7 @@ ARP_Scan_Init(void)
     fclose(in);
     time(&tm);
 #endif                          /* linux */
-#else                           /* CAN_USE_SYSCTL */
+#else                           /* NETSNMP_CAN_USE_SYSCTL */
 
     int             mib[6];
     size_t          needed;
@@ -679,7 +704,7 @@ ARP_Scan_Init(void)
         }
     }
 
-#endif                          /* CAN_USE_SYSCTL */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL */
 }
 
 #ifdef ARP_SCAN_FOUR_ARGUMENTS
@@ -691,7 +716,7 @@ static int
 ARP_Scan_Next(u_long * IPAddr, char *PhysAddr, u_long * ifType)
 #endif
 {
-#ifndef CAN_USE_SYSCTL
+#ifndef NETSNMP_CAN_USE_SYSCTL
 #ifdef linux
     if (arptab_current < arptab_size) {
         /*
@@ -753,8 +778,15 @@ ARP_Scan_Next(u_long * IPAddr, char *PhysAddr, u_long * ifType)
             continue;
         }
 
-        klookup(at_ptr, (char *) &at_entry, sizeof(struct arptab));
-        klookup(at_entry.at_ac, (char *) &at_com, sizeof(struct arpcom));
+        if (!NETSNMP_KLOOKUP(at_ptr, (char *) &at_entry, sizeof(struct arptab))) {
+            DEBUGMSGTL(("mibII/at:ARP_Scan_Next", "klookup failed\n"));
+            break;
+        }
+
+        if (!NETSNMP_KLOOKUP(at_entry.at_ac, (char *) &at_com, sizeof(struct arpcom))) {
+            DEBUGMSGTL(("mibII/at:ARP_Scan_Next", "klookup failed\n"));
+            break;
+        }
 
         at_ptr = at_entry.at_next;
         atab = &at_entry;
@@ -780,7 +812,7 @@ ARP_Scan_Next(u_long * IPAddr, char *PhysAddr, u_long * ifType)
 
     return 0;                   /* we need someone with an irix box to fix this section */
 
-#else                           /* !CAN_USE_SYSCTL */
+#else                           /* !NETSNMP_CAN_USE_SYSCTL */
     struct rt_msghdr *rtm;
     struct sockaddr_inarp *sin;
     struct sockaddr_dl *sdl;
@@ -791,7 +823,11 @@ ARP_Scan_Next(u_long * IPAddr, char *PhysAddr, u_long * ifType)
         sdl = (struct sockaddr_dl *) (sin + 1);
         rtnext += rtm->rtm_msglen;
         if (sdl->sdl_alen) {
+#ifdef irix6
+            *IPAddr = sin->sarp_addr.s_addr;
+#else
             *IPAddr = sin->sin_addr.s_addr;
+#endif
             memcpy(PhysAddr, (char *) LLADDR(sdl), sdl->sdl_alen);
             *ifIndex = sdl->sdl_index;
             *ifType = 1;        /* XXX */
@@ -799,7 +835,7 @@ ARP_Scan_Next(u_long * IPAddr, char *PhysAddr, u_long * ifType)
         }
     }
     return (0);                 /* "EOF" */
-#endif                          /* !CAN_USE_SYSCTL */
+#endif                          /* !NETSNMP_CAN_USE_SYSCTL */
 }
 #endif                          /* solaris2 */
 
@@ -953,7 +989,7 @@ var_atEntry(struct variable *vp,
         free(pIpNetTable);
         return (u_char *) return_buf;
     case IPMEDIANETADDRESS:    /* also ATNETADDRESS */
-        *var_len = sizeof long_return;
+	*var_len = sizeof(uint32_t);
         long_return = pIpNetTable->table[i].dwAddr;
         free(pIpNetTable);
         return (u_char *) & long_return;
