@@ -50,7 +50,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "route_headers.h"
 #define CACHE_TIME (120)        /* Seconds */
 
-#if !defined(CAN_USE_SYSCTL)
+#if !defined(NETSNMP_CAN_USE_SYSCTL)
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -227,6 +227,7 @@ var_ipRouteEntry(struct variable *vp,
 
     switch (vp->magic) {
     case IPROUTEDEST:
+	*var_len = sizeof(uint32_t);
         return (u_char *) get_in_address((struct sockaddr *) (rtp + 1),
                                          rtp->rtm_addrs, RTA_DST);
     case IPROUTEIFINDEX:
@@ -236,30 +237,31 @@ var_ipRouteEntry(struct variable *vp,
         long_return = (rtp->rtm_flags & RTF_UP) ? 1 : 0;
         return (u_char *) & long_return;
     case IPROUTEMETRIC2:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTEMETRIC3:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTEMETRIC4:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTEMETRIC5:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTENEXTHOP:
+	*var_len = sizeof(uint32_t);
         return (u_char *) get_in_address((struct sockaddr *) (rtp + 1),
                                          rtp->rtm_addrs, RTA_GATEWAY);
     case IPROUTETYPE:
@@ -279,7 +281,7 @@ var_ipRouteEntry(struct variable *vp,
             ? 2 : (rtp->rtm_flags & RTF_DYNAMIC) ? 4 : 1;
         return (u_char *) & long_return;
     case IPROUTEAGE:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 0;
@@ -322,7 +324,7 @@ RTENTRY **netsnmp_get_routes(size_t *size) {
 }
 #endif                          /* hpux11 */
 
-#if !(defined(linux) || defined(solaris2) || defined(hpux11))
+#if !(defined(linux) || defined(solaris2) || defined(hpux11)) && defined(RTHOST_SYMBOL) && defined(RTNET_SYMBOL)
 #define NUM_ROUTE_SYMBOLS 2
 static char    *route_symbols[] = {
     RTHOST_SYMBOL,
@@ -402,10 +404,16 @@ static union {
 struct sockaddr_in *
 klgetsa(struct sockaddr_in *dst)
 {
-    klookup((u_long) dst, (char *) &klgetsatmp.sin, sizeof klgetsatmp.sin);
-    if (klgetsatmp.sin.sin_len > sizeof(klgetsatmp.sin))
-        klookup((u_long) dst, (char *) &klgetsatmp.sin,
-                klgetsatmp.sin.sin_len);
+    if (!NETSNMP_KLOOKUP(dst, (char *) &klgetsatmp.sin, sizeof klgetsatmp.sin)) {
+        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+        return NULL;
+    }
+    if (klgetsatmp.sin.sin_len > sizeof(klgetsatmp.sin)) {
+        if (!NETSNMP_KLOOKUP(dst, (char *) &klgetsatmp.sin, klgetsatmp.sin.sin_len)) {
+            DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+            return NULL;
+        }
+    }
     return (&klgetsatmp.sin);
 }
 #endif
@@ -440,7 +448,7 @@ var_ipRouteEntry(struct variable * vp,
      ** this optimisation fails, if there is only a single route avail.
      ** it is a very special case, but better leave it out ...
      **/
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
     saveNameLen = 0;
 #endif
     if (rtsize <= 1)
@@ -530,6 +538,7 @@ var_ipRouteEntry(struct variable * vp,
 
     switch (vp->magic) {
     case IPROUTEDEST:
+	*var_len = sizeof(uint32_t);
 #if NEED_KLGETSA
         sa = klgetsa((struct sockaddr_in *) rthead[RtIndex]->rt_dst);
         return (u_char *) & (sa->sin_addr.s_addr);
@@ -558,7 +567,7 @@ var_ipRouteEntry(struct variable * vp,
 #ifdef hpux11
         long_return = rt[RtIndex].Metric2;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #endif
         long_return = -1;
@@ -567,7 +576,7 @@ var_ipRouteEntry(struct variable * vp,
 #ifdef hpux11
         long_return = rt[RtIndex].Metric3;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #endif
         long_return = -1;
@@ -576,18 +585,19 @@ var_ipRouteEntry(struct variable * vp,
 #ifdef hpux11
         long_return = rt[RtIndex].Metric4;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTEMETRIC5:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = -1;
         return (u_char *) & long_return;
     case IPROUTENEXTHOP:
+	*var_len = sizeof(uint32_t);
 #if NEED_KLGETSA
         sa = klgetsa((struct sockaddr_in *) rthead[RtIndex]->rt_gateway);
         return (u_char *) & (sa->sin_addr.s_addr);
@@ -624,18 +634,19 @@ var_ipRouteEntry(struct variable * vp,
 #ifdef hpux11
         long_return = rt[RtIndex].Age;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #endif
         long_return = 0;
         return (u_char *) & long_return;
     case IPROUTEMASK:
+        *var_len = 4;
 #if NEED_KLGETSA
         /*
          * XXX - Almost certainly not right
          * but I don't have a suitable system to test this on 
          */
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 0;
@@ -648,10 +659,16 @@ var_ipRouteEntry(struct variable * vp,
             long_return = 0;    /* Default route */
         else {
 #ifndef linux
-            klookup((unsigned long) rthead[RtIndex]->rt_ifp,
-                    (char *) &rt_ifnet, sizeof(rt_ifnet));
-            klookup((unsigned long) rt_ifnet.if_addrlist,
-                    (char *) &rt_ifnetaddr, sizeof(rt_ifnetaddr));
+            if (!NETSNMP_KLOOKUP(rthead[RtIndex]->rt_ifp,
+                    (char *) &rt_ifnet, sizeof(rt_ifnet))) {
+                DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                return NULL;
+            }
+            if (!NETSNMP_KLOOKUP(rt_ifnet.if_addrlist,
+                    (char *) &rt_ifnetaddr, sizeof(rt_ifnetaddr))) {
+                DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                return NULL;
+            }
 
             long_return = rt_ifnetaddr.ia_subnetmask;
 #else                           /* linux */
@@ -790,9 +807,16 @@ var_ipRouteEntry(struct variable * vp,
         ipaddr_return = Lowentry.ipRouteDest;
         return (u_char *) & ipaddr_return;
     case IPROUTEIFINDEX:
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+        Lowentry.ipRouteIfIndex.o_bytes[Lowentry.ipRouteIfIndex.o_length] = '\0';
         long_return =
-            Interface_Index_By_Name(Lowentry.ipRouteIfIndex.o_bytes,
-                                    Lowentry.ipRouteIfIndex.o_length);
+            netsnmp_access_interface_index_find(
+                Lowentry.ipRouteIfIndex.o_bytes);
+#else
+        long_return =
+           Interface_Index_By_Name(Lowentry.ipRouteIfIndex.o_bytes,
+                                   Lowentry.ipRouteIfIndex.o_length);
+#endif
         return (u_char *) & long_return;
     case IPROUTEMETRIC1:
         long_return = Lowentry.ipRouteMetric1;
@@ -852,9 +876,8 @@ load_rtentries(struct radix_node *pt)
     register char  *cp;
 #endif
 
-    if (!klookup
-        ((unsigned long) pt, (char *) &node, sizeof(struct radix_node))) {
-        DEBUGMSGTL(("mibII/var_route", "Fail\n"));
+    if (!NETSNMP_KLOOKUP(pt, (char *) &node, sizeof(struct radix_node))) {
+        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
         return;
     }
     if (node.rn_b >= 0) {
@@ -872,20 +895,31 @@ load_rtentries(struct radix_node *pt)
         /*
          * get the route 
          */
-        klookup((unsigned long) pt, (char *) &rt, sizeof(RTENTRY));
+        if (!NETSNMP_KLOOKUP(pt, (char *) &rt, sizeof(RTENTRY))) {
+            DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+            return;
+        }
 
         if (rt.rt_ifp != 0) {
-            klookup((unsigned long) rt.rt_ifp, (char *) &ifnet,
-                    sizeof(ifnet));
+            if (!NETSNMP_KLOOKUP(rt.rt_ifp, (char *) &ifnet, sizeof(ifnet))) {
+                DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                return;
+            }
 #if STRUCT_IFNET_HAS_IF_XNAME
 #if defined(netbsd1) || defined(openbsd2)
             strncpy(name, ifnet.if_xname, sizeof name);
 #else
-            klookup((unsigned long) ifnet.if_xname, name, sizeof name);
+            if (!NETSNMP_KLOOKUP(ifnet.if_xname, name, sizeof name)) {
+                DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                return;
+            }
 #endif
             name[sizeof(name) - 1] = '\0';
 #else
-            klookup((unsigned long) ifnet.if_name, name, sizeof name);
+            if (!NETSNMP_KLOOKUP(ifnet.if_name, name, sizeof name)) {
+                DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                return;
+            }
             name[sizeof(name) - 1] = '\0';
             cp = (char *) strchr(name, '\0');
             string_append_int(cp, ifnet.if_unit);
@@ -1033,8 +1067,7 @@ Route_Scan_Reload(void)
     for (i = 0; i <= AF_MAX; i++) {
         if (rt_table[i] == 0)
             continue;
-        if (klookup
-            ((unsigned long) rt_table[i], (char *) &head, sizeof(head))) {
+        if (NETSNMP_KLOOKUP(rt_table[i], (char *) &head, sizeof(head))) {
             load_rtentries(head.rnh_treetop);
         }
     }
@@ -1054,13 +1087,22 @@ Route_Scan_Reload(void)
                 /*
                  *      Dig the route out of the kernel...
                  */
-                klookup(m, (char *) &mb, sizeof(mb));
+                if (!NETSNMP_KLOOKUP(m, (char *) &mb, sizeof(mb))) {
+                    DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                    return;
+                }
                 m = mb.rt_next;
 
                 rt = &mb;
                 if (rt->rt_ifp != 0) {
-                    klookup(rt->rt_ifp, (char *) &ifnet, sizeof(ifnet));
-                    klookup(ifnet.if_name, name, 16);
+                    if (!NETSNMP_KLOOKUP(rt->rt_ifp, (char *) &ifnet, sizeof(ifnet))) {
+                        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                        return;
+                    }
+                    if (!NETSNMP_KLOOKUP(ifnet.if_name, name, 16)) {
+                        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                        return;
+                    }
                     name[15] = '\0';
                     cp = (char *) strchr(name, '\0');
                     string_append_int(cp, ifnet.if_unit);
@@ -1171,15 +1213,23 @@ Route_Scan_Reload(void)
                 /*
                  *  Dig the route out of the kernel...
                  */
-                klookup((unsigned long) m, (char *) &mb, sizeof(mb));
+                if (!NETSNMP_KLOOKUP(m, (char *) &mb, sizeof(mb))) {
+                    DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                    return;
+                }
                 m = mb.m_next;
                 rt = mtod(&mb, RTENTRY *);
 
                 if (rt->rt_ifp != 0) {
 
-                    klookup((unsigned long) rt->rt_ifp, (char *) &ifnet,
-                            sizeof(ifnet));
-                    klookup((unsigned long) ifnet.if_name, name, 16);
+                    if (!NETSNMP_KLOOKUP(rt->rt_ifp, (char *) &ifnet, sizeof(ifnet))) {
+                        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                        return;
+                    }
+                    if (!NETSNMP_KLOOKUP(ifnet.if_name, name, 16)) {
+                        DEBUGMSGTL(("mibII/var_route", "klookup failed\n"));
+                        return;
+                    }
                     name[15] = '\0';
                     cp = (char *) strchr(name, '\0');
                     string_append_int(cp, ifnet.if_unit);
@@ -1433,7 +1483,7 @@ var_ipRouteEntry(struct variable *vp,
      ** this optimisation fails, if there is only a single route avail.
      ** it is a very special case, but better leave it out ...
      **/
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
     saveNameLen = 0;
 #endif
     if (route_row == NULL) {
@@ -1558,6 +1608,7 @@ var_ipRouteEntry(struct variable *vp,
 
     switch (vp->magic) {
     case IPROUTEDEST:
+	*var_len = sizeof(uint32_t);
         *write_method = write_rte;
         long_return = pIpRtrTable->table[RtIndex].dwForwardDest;
         return (u_char *) & long_return;
@@ -1586,6 +1637,7 @@ var_ipRouteEntry(struct variable *vp,
         long_return = pIpRtrTable->table[RtIndex].dwForwardMetric5;
         return (u_char *) & long_return;
     case IPROUTENEXTHOP:
+	*var_len = sizeof(uint32_t);
         *write_method = write_rte;
         long_return = pIpRtrTable->table[RtIndex].dwForwardNextHop;
         return (u_char *) & long_return;
@@ -1616,7 +1668,7 @@ var_ipRouteEntry(struct variable *vp,
 
 #endif                          /* WIN32 cygwin */
 
-#else                           /* CAN_USE_SYSCTL */
+#else                           /* NETSNMP_CAN_USE_SYSCTL */
 
 static
 TAILQ_HEAD(, snmprt)
@@ -1849,10 +1901,11 @@ var_ipRouteEntry(struct variable * vp,
         *length = 14;
     }
 
-    *var_len = sizeof long_return;
+    *var_len = sizeof(long_return);
 
     switch (vp->magic) {
     case IPROUTEDEST:
+	*var_len = sizeof(uint32_t);
         long_return = rt->dest.s_addr;
         return (u_char *) & long_return;
 
@@ -1877,6 +1930,7 @@ var_ipRouteEntry(struct variable * vp,
         return (u_char *) & long_return;
 
     case IPROUTENEXTHOP:
+	*var_len = sizeof(uint32_t);
         if (rt->gateway.s_addr == 0 && rt->ifa.s_addr == 0)
             long_return = 0;
         else if (rt->gateway.s_addr == 0)
@@ -1902,7 +1956,7 @@ var_ipRouteEntry(struct variable * vp,
         return (u_char *) & long_return;
 
     case IPROUTEAGE:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 0;
@@ -1928,7 +1982,7 @@ init_var_route(void)
     ;
 }
 
-#endif                          /* CAN_USE_SYSCTL */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL */
 
 #if defined(HAVE_SYS_SYSCTL_H) && !defined(linux)
 /*
