@@ -1,7 +1,7 @@
 /*
  *  TcpConn MIB architecture support
  *
- * $Id: tcpConn_common.c,v 1.4 2005/12/11 17:55:01 rstory Exp $
+ * $Id: tcpConn_common.c 16581 2007-07-02 05:31:33Z magfr $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -16,8 +16,6 @@
 /*
  * local static prototypes
  */
-static int _access_tcpconn_entry_compare_addr(const void *lhs,
-                                                const void *rhs);
 static void _access_tcpconn_entry_release(netsnmp_tcpconn_entry * entry,
                                             void *unused);
 
