@@ -154,7 +154,7 @@ static void       add_device(char *path, char *device,
 static void       modify_disk_parameters(int index, int minspace,
 	                                 int minpercent);
 static int        disk_exists(char *path);
-static u_char    *find_device(char *path);
+static char      *find_device(char *path);
 
 struct diskpart {
     char            device[STRMAX];
@@ -192,7 +192,7 @@ struct variable2 extensible_disk_variables[] = {
  * Define the OID pointer to the top of the mib tree that we're
  * registering underneath 
  */
-oid             disk_variables_oid[] = { UCDAVIS_MIB, DISKMIBNUM, 1 };
+oid             disk_variables_oid[] = { NETSNMP_UCDAVIS_MIB, NETSNMP_DISKMIBNUM, 1 };
 
 void
 init_disk(void)
@@ -224,6 +224,7 @@ disk_free_config(void)
     disks[i].minimumspace = -1;
     disks[i].minpercent = -1;
   }
+  allDisksIncluded = 0;
 }
 
 static void 
@@ -281,7 +282,7 @@ disk_parse_config(const char *token, char *cptr)
           minpercent = atoi(cptr);
       }
   } else {
-      minspace = DEFDISKMINIMUMSPACE;
+      minspace = NETSNMP_DEFDISKMINIMUMSPACE;
       minpercent = -1;
   }
 
@@ -355,6 +356,10 @@ add_device(char *path, char *device, int minspace, int minpercent, int override)
 {
   int index;
 
+  if (!path || !strcmp(path, "none")) {
+    DEBUGMSGTL(("ucd-snmp/disk", "Skipping null path device (%s)\n", device));
+    return;
+  }
   if (numdisks == maxdisks) {
       if (maxdisks == 0) {
           maxdisks = 50;
@@ -417,7 +422,7 @@ int disk_exists(char *path)
 {
   int index;
   for(index = 0; index < numdisks; index++) {
-    DEBUGMSGTL(("ucd-snmp/disk:", "Checking for %s. Found %s at %d ", path, disks[index].path, index));
+    DEBUGMSGTL(("ucd-snmp/disk", "Checking for %s. Found %s at %d\n", path, disks[index].path, index));
     if(strcmp(path, disks[index].path) == 0) {
       return index;
     }
@@ -519,7 +524,7 @@ find_and_add_allDisks(int minpercent)
  
 }
 
-static u_char *
+static char *
 find_device(char *path)
 {
 #if HAVE_GETMNTENT
@@ -596,7 +601,7 @@ find_device(char *path)
 
 #elif HAVE_STATFS
   if (statfs(path, &statf) == 0) {
-    copy_word(statf.f_mntfromname, device);
+    copy_nword(statf.f_mntfromname, device, sizeof(device));
     DEBUGMSGTL(("ucd-snmp/disk", "Disk:  %s\n",
 		statf.f_mntfromname));
   }
@@ -665,6 +670,7 @@ var_extensible_disk(struct variable *vp,
 #endif
 #endif
 
+tryAgain:
     if (header_simple_table
         (vp, name, length, exact, var_len, write_method, numdisks))
         return (NULL);
@@ -696,6 +702,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Couldn't open device %s\n",
                  disks[disknum].device);
         setPerrorstatus("statvfs dev/disk");
+        if (!exact)
+            goto tryAgain;
         return NULL;
     }
 #ifdef STAT_STATFS_FS_DATA
@@ -709,7 +717,9 @@ var_extensible_disk(struct variable *vp,
     vfs.f_bfree = vfs.f_spare[1];
     vfs.f_bavail = vfs.f_spare[2];
 #endif
-    percent = vfs.f_bavail <= 0 ? 100 :
+    percent =
+        vfs.f_blocks == 0 ? 0 :
+        vfs.f_bavail <= 0 ? 100 :
         (int) ((double) (vfs.f_blocks - vfs.f_bfree) /
                (double) (vfs.f_blocks -
                          (vfs.f_bfree - vfs.f_bavail)) * 100.0 + 0.5);
@@ -781,6 +791,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Couldn't open device %s\n",
                  disks[disknum].device);
         setPerrorstatus("open dev/disk");
+        if (!exact)
+            goto tryAgain;
         return (NULL);
     }
     lseek(file, (long) (SBLOCK * DEV_BSIZE), 0);
@@ -789,6 +801,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Error reading device %s\n",
                  disks[disknum].device);
         close(file);
+        if (!exact)
+            goto tryAgain;
         return (NULL);
     }
     close(file);
@@ -799,8 +813,9 @@ var_extensible_disk(struct variable *vp,
     availblks = totalblks * (100 - filesys.fs_minfree) / 100;
     avail = availblks > used ? availblks - used : 0;
     percent =
-        availblks ==
-        0 ? 100 : (int) ((double) used / (double) totalblks * 100.0 + 0.5);
+        totalblks == 0 ? 0 :
+        availblks == 0 ? 100 :
+        (int) ((double) used / (double) totalblks * 100.0 + 0.5);
     multiplier = (float)filesys.fs_fsize / (float)1024.0;
     iserror =
         (disks[disknum].minimumspace >= 0
