@@ -282,12 +282,12 @@ var_diskio(struct variable * vp,
         long_ret = (uint32_t) kio.nwritten;
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = kio.nread & 0xffffffff;;
         c64_ret.high = kio.nread >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = kio.nwritten & 0xffffffff;;
         c64_ret.high = kio.nwritten >> 32;
         return (u_char *) & c64_ret;
@@ -608,7 +608,7 @@ var_diskio(struct variable * vp,
 #endif
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
 #if defined(freebsd5) && __FreeBSD_version >= 500107
         longlong_ret = stat->dinfo->devices[indx].bytes[DEVSTAT_READ];
 #else
@@ -618,7 +618,7 @@ var_diskio(struct variable * vp,
         c64_ret.high = longlong_ret >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
 #if defined(freebsd5) && __FreeBSD_version >= 500107
         longlong_ret = stat->dinfo->devices[indx].bytes[DEVSTAT_WRITE];
 #else
@@ -737,6 +737,7 @@ int getstats(void)
     else {
 	/* See if a 2.4 kernel */
 	char buffer[1024];
+	int rc;
 	parts = fopen("/proc/partitions", "r");
 	if (!parts) {
 	    snmp_log_perror("/proc/partitions");
@@ -758,11 +759,15 @@ int getstats(void)
 	    }
 	    pTemp = &head.indices[head.length];
 
-	    fscanf (parts, "%d %d %lu %s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
+	    rc = fscanf (parts, "%d %d %lu %s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
 		    &pTemp->major, &pTemp->minor, &pTemp->blocks, pTemp->name,
 		    &pTemp->rio, &pTemp->rmerge, &pTemp->rsect, &pTemp->ruse,
 		    &pTemp->wio, &pTemp->wmerge, &pTemp->wsect, &pTemp->wuse,
 		    &pTemp->running, &pTemp->use, &pTemp->aveq);
+            if (rc != 15) {
+               snmp_log(LOG_ERR, "diskio.c: cannot find statistics in /proc/partitions\n");
+               return 1;
+            }
 	    head.length++;
 	}
     }
@@ -818,12 +823,12 @@ var_diskio(struct variable * vp,
       long_ret = head.indices[indx].wio;
       return (u_char *) & long_ret;
     case DISKIO_NREADX:
-      *var_len = 8;
+      *var_len = sizeof(struct counter64);
       c64_ret.low = head.indices[indx].rsect * 512 & 0xffffffff;
       c64_ret.high = head.indices[indx].rsect >> (32 - 9);
       return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-      *var_len = 8;
+      *var_len = sizeof(struct counter64);
       c64_ret.low = head.indices[indx].wsect * 512 & 0xffffffff;
       c64_ret.high = head.indices[indx].wsect >> (32 - 9);
       return (u_char *) & c64_ret;
@@ -927,7 +932,7 @@ static int
 handle_drive(io_registry_entry_t drive, struct drivestats * dstat)
 {
     io_registry_entry_t parent;
-    CFDictionaryRef     properties;
+    CFMutableDictionaryRef     properties;
     CFStringRef         name;
     CFNumberRef         number;
     kern_return_t       status;
@@ -943,7 +948,7 @@ handle_drive(io_registry_entry_t drive, struct drivestats * dstat)
     if (IOObjectConformsTo(parent, "IOBlockStorageDriver")) {
 
 	/* get drive properties */
-	status = IORegistryEntryCreateCFProperties(drive, (CFMutableDictionaryRef *)&properties,
+	status = IORegistryEntryCreateCFProperties(drive, &properties,
 					    kCFAllocatorDefault, kNilOptions);
 	if (status != KERN_SUCCESS) {
 	    snmp_log(LOG_ERR, "diskio: device has no properties\n");
@@ -958,7 +963,7 @@ handle_drive(io_registry_entry_t drive, struct drivestats * dstat)
 					    CFSTR(kIOBSDUnitKey));
 
 	/* Collect stats and if succesful store them with the name and unitnumber */
-	if (!collect_drive_stats(parent, dstat->stats)) {
+	if (name && number && !collect_drive_stats(parent, dstat->stats)) {
 
 	    CFStringGetCString(name, dstat->name, MAXDRIVENAME, CFStringGetSystemEncoding());
 	    CFNumberGetValue(number, kCFNumberSInt32Type, &dstat->bsd_unit_number);
@@ -1144,12 +1149,12 @@ var_diskio(struct variable * vp,
         long_ret = (signed long) 0;	/* AIX has just one value for read/write transfers */
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = (ps_disk[indx].rblks * ps_disk[indx].bsize) & 0xffffffff;;
         c64_ret.high = (ps_disk[indx].rblks * ps_disk[indx].bsize) >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = (ps_disk[indx].wblks * ps_disk[indx].bsize) & 0xffffffff;;
         c64_ret.high = (ps_disk[indx].wblks * ps_disk[indx].bsize) >> 32;
         return (u_char *) & c64_ret;
