@@ -25,13 +25,13 @@ netsnmp_parse_iqueryVersion(const char *token, char *line)
 {
     char buf[1024];
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (!strcmp( line, "1" ))
         netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
                            NETSNMP_DS_AGENT_INTERNAL_VERSION, SNMP_VERSION_1);
     else 
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
          if (!strcmp( line, "2"  ) || !strcasecmp( line, "2c" ))
         netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
                            NETSNMP_DS_AGENT_INTERNAL_VERSION, SNMP_VERSION_2c);
@@ -62,10 +62,10 @@ _init_default_iquery_session( int majorID, int minorID,
 void init_iquery(void){
     char *type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
                                        NETSNMP_DS_LIB_APPTYPE);
-    netsnmp_ds_register_config(ASN_OCTET_STR, type, "agentSecName",
+    netsnmp_ds_register_premib(ASN_OCTET_STR, type, "agentSecName",
                                NETSNMP_DS_APPLICATION_ID,
                                NETSNMP_DS_AGENT_INTERNAL_SECNAME);
-    netsnmp_ds_register_config(ASN_OCTET_STR, type, "iquerySecName",
+    netsnmp_ds_register_premib(ASN_OCTET_STR, type, "iquerySecName",
                                NETSNMP_DS_APPLICATION_ID,
                                NETSNMP_DS_AGENT_INTERNAL_SECNAME);
 
@@ -85,7 +85,7 @@ void init_iquery(void){
                        NETSNMP_DS_AGENT_INTERNAL_SECLEVEL, SNMP_SEC_LEVEL_AUTHNOPRIV);
 
     snmp_register_callback(SNMP_CALLBACK_LIBRARY, 
-                           SNMP_CALLBACK_POST_READ_CONFIG,
+                           SNMP_CALLBACK_POST_PREMIB_READ_CONFIG,
                            _init_default_iquery_session, NULL);
 }
 
@@ -144,7 +144,7 @@ netsnmp_session *netsnmp_iquery_session(char* secName,   int   version,
     extern int callback_master_num;
     netsnmp_session *ss = NULL;
 
-#ifdef SNMP_TRANSPORT_CALLBACK_DOMAIN
+#ifdef NETSNMP_TRANSPORT_CALLBACK_DOMAIN
     ss = netsnmp_callback_open( callback_master_num, NULL, NULL, NULL);
     if (ss) {
         ss->version       = version;
