@@ -210,17 +210,20 @@ main(int argc, char *argv[])
                 }
                 if (vars->name_length == length_sysUpTime &&
                     !memcmp(objid_sysUpTime, vars->name,
-                            sizeof(objid_sysUpTime))) {
+                            sizeof(objid_sysUpTime)) &&
+                    vars->val.integer) {
                     uptime = *vars->val.integer;
                 }
                 if (vars->name_length == length_ipInReceives &&
                     !memcmp(objid_ipInReceives, vars->name,
-                            sizeof(objid_ipInReceives))) {
+                            sizeof(objid_ipInReceives)) &&
+                    vars->val.integer) {
                     ipin = *vars->val.integer;
                 }
                 if (vars->name_length == length_ipOutRequests &&
                     !memcmp(objid_ipOutRequests, vars->name,
-                            sizeof(objid_ipOutRequests))) {
+                            sizeof(objid_ipOutRequests)) &&
+                    vars->val.integer) {
                     ipout = *vars->val.integer;
                 }
             }
