@@ -156,7 +156,7 @@ main(int argc, char *argv[])
     char           *trap = NULL;
     char           *prognam;
     int             exitval = 0;
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     char           *specific = NULL, *description = NULL, *agent = NULL;
     in_addr_t      *pdu_in_addr_t;
 #endif
@@ -185,8 +185,6 @@ main(int argc, char *argv[])
 
     session.callback = snmp_input;
     session.callback_magic = NULL;
-    netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_DEFAULT_PORT, 
-		       SNMP_TRAP_PORT);
 
     if (session.version == SNMP_VERSION_3 && !inform) {
         /*
@@ -239,17 +237,20 @@ main(int argc, char *argv[])
             session.engineTime = get_uptime();  /* but it'll work. Sort of. */
     }
 
-    ss = snmp_open(&session);
+    ss = snmp_add(&session,
+                  netsnmp_transport_open_client("snmptrap", session.peername),
+                  NULL, NULL);
     if (ss == NULL) {
         /*
-         * diagnose snmp_open errors with the input netsnmp_session pointer 
+         * diagnose netsnmp_transport_open_client and snmp_add errors with
+         * the input netsnmp_session pointer
          */
         snmp_sess_perror("snmptrap", &session);
         SOCK_CLEANUP;
         exit(1);
     }
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (session.version == SNMP_VERSION_1) {
         if (inform) {
             fprintf(stderr, "Cannot send INFORM as SNMPv1 PDU\n");
