@@ -87,6 +87,12 @@ SOFTWARE.
 #if HAVE_PROCESS_H              /* Win32-getpid */
 #include <process.h>
 #endif
+#if HAVE_PWD_H
+#include <pwd.h>
+#endif
+#if HAVE_GRP_H
+#include <grp.h>
+#endif
 #include <signal.h>
 #include <errno.h>
 
@@ -97,6 +103,7 @@ SOFTWARE.
 #include "snmptrapd_log.h"
 #include "snmptrapd_auth.h"
 #include "notification-log-mib/notification_log.h"
+#include "mibII/vacm_conf.h"
 
 /*
  * Include winservice.h to support Windows Service
@@ -110,7 +117,7 @@ SOFTWARE.
 
 #endif
 
-#if USE_LIBWRAP
+#if NETSNMP_USE_LIBWRAP
 #include <tcpd.h>
 #endif
 
@@ -217,7 +224,7 @@ int __cdecl     _tmain(int argc, TCHAR * argv[]);
 int             main(int, char **);
 #endif
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
 extern void            subagent_init(void);
 #endif
 
@@ -302,6 +309,7 @@ usage(void)
     fprintf(stderr, "\tEmail:    net-snmp-coders@lists.sourceforge.net\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "  -a\t\t\tignore authentication failure traps\n");
+    fprintf(stderr, "  -A\t\t\tappend to log file rather than truncating it\n");
     fprintf(stderr, "  -c FILE\t\tread FILE as a configuration file\n");
     fprintf(stderr,
             "  -C\t\t\tdo not read the default configuration files\n");
@@ -312,6 +320,10 @@ usage(void)
     fprintf(stderr, "  -f\t\t\tdo not fork from the shell\n");
     fprintf(stderr,
             "  -F FORMAT\t\tuse specified format for logging to standard error\n");
+#if HAVE_UNISTD_H
+    fprintf(stderr, "  -g GID\t\tchange to this numeric gid after opening\n"
+	   "\t\t\t  transport endpoints\n");
+#endif
     fprintf(stderr, "  -h, --help\t\tdisplay this usage message\n");
     fprintf(stderr,
             "  -H\t\t\tdisplay configuration file directives understood\n");
@@ -331,12 +343,16 @@ usage(void)
     fprintf(stderr, "  \t\t\t  Note that some parameters are not relevant when running as a service\n");
 #endif
     fprintf(stderr, "  -t\t\t\tPrevent traps from being logged to syslog\n");
+#if HAVE_UNISTD_H
+    fprintf(stderr, "  -u UID\t\tchange to this uid (numeric or textual) after\n"
+	   "\t\t\t  opening transport endpoints\n");
+#endif
 #ifdef WIN32SERVICE
     fprintf(stderr, "  -unregister\t\tunregister as a Windows service\n");
     fprintf(stderr, "  \t\t\t  (followed -quiet to prevent message popups)\n");
 #endif
     fprintf(stderr, "  -v, --version\t\tdisplay version information\n");
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     fprintf(stderr, "  -x ADDRESS\t\tuse ADDRESS as AgentX address\n");
 #endif
     fprintf(stderr,
@@ -350,7 +366,6 @@ usage(void)
     fprintf(stderr, "  -P\t\t\tuse -f -Le  instead\n");
     fprintf(stderr, "  -s\t\t\tuse -Lsd instead\n");
     fprintf(stderr, "  -S d|i|0-7\t\tuse -Ls <facility> instead\n");
-    fprintf(stderr, "  -u FILE\t\tuse -p <FILE> instead\n");
 }
 
 static void
@@ -392,7 +407,7 @@ static int
 pre_parse(netsnmp_session * session, netsnmp_transport *transport,
           void *transport_data, int transport_data_length)
 {
-#if USE_LIBWRAP
+#if NETSNMP_USE_LIBWRAP
     char *addr_string = NULL;
 
     if (transport != NULL && transport->f_fmtaddr != NULL) {
@@ -433,7 +448,7 @@ pre_parse(netsnmp_session * session, netsnmp_transport *transport,
             return 0;
         }
     }
-#endif/*  USE_LIBWRAP  */
+#endif/*  NETSNMP_USE_LIBWRAP  */
     return 1;
 }
 
@@ -517,6 +532,65 @@ parse_config_pidFile(const char *token, char *cptr)
   pid_file = strdup (cptr);
 }
 
+#ifdef HAVE_UNISTD_H
+void
+parse_config_agentuser(const char *token, char *cptr)
+{
+#if defined(HAVE_GETPWNAM) && defined(HAVE_PWD_H)
+    struct passwd  *info;
+#endif
+
+    if (cptr[0] == '#') {
+        char           *ecp;
+        int             uid;
+        uid = strtoul(cptr + 1, &ecp, 10);
+        if (*ecp != 0) {
+            config_perror("Bad number");
+	} else {
+	    netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+			       NETSNMP_DS_AGENT_USERID, uid);
+	}
+    }
+#if defined(HAVE_GETPWNAM) && defined(HAVE_PWD_H)
+    else if ((info = getpwnam(cptr)) != NULL) {
+        netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+			   NETSNMP_DS_AGENT_USERID, info->pw_uid);
+    } else {
+        config_perror("User not found in passwd database");
+    }
+    endpwent();
+#endif
+}
+
+void
+parse_config_agentgroup(const char *token, char *cptr)
+{
+#if defined(HAVE_GETGRNAM) && defined(HAVE_GRP_H)
+    struct group   *info;
+#endif
+
+    if (cptr[0] == '#') {
+        char           *ecp;
+        int             gid = strtoul(cptr + 1, &ecp, 10);
+        if (*ecp != 0) {
+            config_perror("Bad number");
+	} else {
+            netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+			       NETSNMP_DS_AGENT_GROUPID, gid);
+	}
+    }
+#if defined(HAVE_GETGRNAM) && defined(HAVE_GRP_H)
+    else if ((info = getgrnam(cptr)) != NULL) {
+        netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+			   NETSNMP_DS_AGENT_GROUPID, info->gr_gid);
+    } else {
+        config_perror("Group not found in group database");
+    }
+    endpwent();
+#endif
+}
+#endif
+
 void
 parse_config_logOption(const char *token, char *cptr)
 {
@@ -583,24 +657,28 @@ SnmpTrapdMain(int argc, TCHAR * argv[])
 main(int argc, char *argv[])
 #endif
 {
-    char            options[128] = "ac:CdD::efF:hHI:L:m:M:no:PqsS:tvx:O:-:";
+    char            options[128] = "aAc:CdD::efF:g:hHI:L:m:M:no:O:PqsS:tu:vx:-:";
     netsnmp_session *sess_list = NULL, *ss = NULL;
     netsnmp_transport *transport = NULL;
     int             arg, i = 0, depmsg = 0;
+    int             uid = 0, gid = 0;
     int             count, numfds, block;
     fd_set          readfds,writefds,exceptfds;
     struct timeval  timeout, *tvp;
     char           *cp, *listen_ports = NULL;
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     int             agentx_subagent = 1;
 #endif
     netsnmp_trapd_handler *traph;
+#ifdef NETSNMP_EMBEDDED_PERL
+    extern void init_perl(void);
+#endif
 
 #ifdef SIGTERM
     signal(SIGTERM, term_handler);
 #endif
 #ifdef SIGHUP
-    signal(SIGHUP, hup_handler);
+    signal(SIGHUP, SIG_IGN);   /* do not terminate on early SIGHUP */
 #endif
 #ifdef SIGINT
     signal(SIGINT, term_handler);
@@ -609,6 +687,10 @@ main(int argc, char *argv[])
     signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
 #endif
 
+#ifdef SIGHUP
+    signal(SIGHUP, SIG_IGN);   /* do not terminate on early SIGHUP */
+#endif
+
     /*
      * register our configuration handlers now so -H properly displays them 
      */
@@ -623,6 +705,12 @@ main(int argc, char *argv[])
     register_config_handler("snmptrapd", "pidFile",
                             parse_config_pidFile, NULL, "string");
 #endif
+#ifdef HAVE_UNISTD_H
+    register_config_handler("snmptrapd", "agentuser",
+                            parse_config_agentuser, NULL, "userid");
+    register_config_handler("snmptrapd", "agentgroup",
+                            parse_config_agentgroup, NULL, "groupid");
+#endif
     
     register_config_handler("snmptrapd", "logOption",
                             parse_config_logOption, NULL, "string");
@@ -649,7 +737,7 @@ main(int argc, char *argv[])
      * Add some options if they are available.  
      */
 #if HAVE_GETPID
-    strcat(options, "p:u:");
+    strcat(options, "p:");
 #endif
 
 #ifdef WIN32
@@ -727,6 +815,18 @@ main(int argc, char *argv[])
             }
             break;
 
+#if HAVE_UNISTD_H
+        case 'g':
+            if (optarg != NULL) {
+                netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+				   NETSNMP_DS_AGENT_GROUPID, gid = atoi(optarg));
+            } else {
+                usage();
+                exit(1);
+            }
+            break;
+#endif
+
         case 'h':
             usage();
             exit(0);
@@ -856,6 +956,17 @@ main(int argc, char *argv[])
             }
             break;
 
+#if HAVE_GETPID
+        case 'p':
+            if (optarg != NULL) {
+                parse_config_pidFile(NULL, optarg);
+            } else {
+                usage();
+                exit(1);
+            }
+            break;
+#endif
+
         case 'P':
             fprintf(stderr,
                     "Warning: -P option is deprecated; use -f -Le instead\n");
@@ -878,14 +989,28 @@ main(int argc, char *argv[])
             SyslogTrap++;
             break;
 
-
-#if HAVE_GETPID
+#if HAVE_UNISTD_H
         case 'u':
-            fprintf(stderr,
-                    "Warning: -u option is deprecated; use -p instead\n");
-        case 'p':
             if (optarg != NULL) {
-                parse_config_pidFile(NULL, optarg);
+                char           *ecp;
+
+                uid = strtoul(optarg, &ecp, 10);
+                if (*ecp) {
+#if HAVE_GETPWNAM && HAVE_PWD_H
+                    struct passwd  *info;
+                    info = getpwnam(optarg);
+                    if (info) {
+                        uid = info->pw_uid;
+                    } else {
+#endif
+                        fprintf(stderr, "Bad user id: %s\n", optarg);
+                        exit(1);
+#if HAVE_GETPWNAM && HAVE_PWD_H
+                    }
+#endif
+                }
+                netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
+				   NETSNMP_DS_AGENT_USERID, uid);
             } else {
                 usage();
                 exit(1);
@@ -950,6 +1075,7 @@ main(int argc, char *argv[])
         traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
                                                syslog_handler);
         traph->authtypes = TRAP_AUTH_LOG;
+        snmp_enable_syslog();
     } else {
         traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
                                                print_handler);
@@ -974,7 +1100,7 @@ main(int argc, char *argv[])
 	 */
     }
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     /*
      * we're an agentx subagent? 
      */
@@ -1003,16 +1129,13 @@ main(int argc, char *argv[])
      */
     init_agent("snmptrapd");
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     /*
      * initialize local modules 
      */
     if (agentx_subagent) {
 #ifdef USING_SNMPV3_SNMPENGINE_MODULE
         extern void register_snmpEngine_scalars_context(const char *);
-#endif
-#ifdef USING_SNMPV3_USMUSER_MODULE
-        extern void init_register_usmUser_context(const char *);
 #endif
         subagent_init();
 #ifdef USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE
@@ -1035,13 +1158,25 @@ main(int argc, char *argv[])
          */
         register_snmpEngine_scalars_context("snmptrapd");
 #endif
+    }
+#endif /* USING_AGENTX_SUBAGENT_MODULE && !NETSNMP_SNMPTRAPD_DISABLE_AGENTX */
 
-        /* register our authorization handler */
-        init_netsnmp_trapd_auth();
+    /* register our authorization handler */
+    init_netsnmp_trapd_auth();
 
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
+    if (agentx_subagent) {
+#ifdef USING_AGENT_NSVACMACCESSTABLE_MODULE
+        extern void init_register_nsVacm_context(const char *);
+#endif
 #ifdef USING_SNMPV3_USMUSER_MODULE
+        extern void init_register_usmUser_context(const char *);
         /* register ourselves as having a USM user database */
         init_register_usmUser_context("snmptrapd");
+#endif
+#ifdef USING_AGENT_NSVACMACCESSTABLE_MODULE
+        /* register net-snmp vacm extensions */
+        init_register_nsVacm_context("snmptrapd");
 #endif
     }
 #endif
@@ -1064,6 +1199,10 @@ main(int argc, char *argv[])
      */
     init_snmp("snmptrapd");
 
+#ifdef SIGHUP
+    signal(SIGHUP, hup_handler);
+#endif
+
     if (trap1_fmt_str_remember) {
         free_trap1_fmt();
         free_trap2_fmt();
@@ -1153,26 +1292,15 @@ main(int argc, char *argv[])
 
     while (cp != NULL) {
         char *sep = strchr(cp, ',');
-        char  listen_name[128];
-        char *cp2 = strchr(cp, ':');
 
         if (sep != NULL) {
             *sep = 0;
         }
 
-           /*
-            * Make sure this defaults to listening on port 162
-            */
-        if (!cp2) {
-            snprintf(listen_name, sizeof(listen_name), "%s:162", cp);
-            cp2 = listen_name;
-        } else {
-            cp2 = cp;
-        }
-        transport = netsnmp_tdomain_transport(cp2, 1, "udp");
+        transport = netsnmp_transport_open_server("snmptrap", cp);
         if (transport == NULL) {
             snmp_log(LOG_ERR, "couldn't open %s -- errno %d (\"%s\")\n",
-                     cp2, errno, strerror(errno));
+                     cp, errno, strerror(errno));
             snmptrapd_close_sessions(sess_list);
             SOCK_CLEANUP;
             exit(1);
@@ -1211,6 +1339,39 @@ main(int argc, char *argv[])
      */
     reconfig = 0;
 
+#if HAVE_UNISTD_H
+#ifdef HAVE_SETGID
+    if ((gid = netsnmp_ds_get_int(NETSNMP_DS_APPLICATION_ID, 
+				  NETSNMP_DS_AGENT_GROUPID)) != 0) {
+        DEBUGMSGTL(("snmptrapd/main", "Changing gid to %d.\n", gid));
+        if (setgid(gid) == -1
+#ifdef HAVE_SETGROUPS
+            || setgroups(1, (gid_t *)&gid) == -1
+#endif
+            ) {
+            snmp_log_perror("setgid failed");
+            if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
+					NETSNMP_DS_AGENT_NO_ROOT_ACCESS)) {
+                exit(1);
+            }
+        }
+    }
+#endif
+#ifdef HAVE_SETUID
+    if ((uid = netsnmp_ds_get_int(NETSNMP_DS_APPLICATION_ID, 
+				  NETSNMP_DS_AGENT_USERID)) != 0) {
+        DEBUGMSGTL(("snmptrapd/main", "Changing uid to %d.\n", uid));
+        if (setuid(uid) == -1) {
+            snmp_log_perror("setuid failed");
+            if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
+					NETSNMP_DS_AGENT_NO_ROOT_ACCESS)) {
+                exit(1);
+            }
+        }
+    }
+#endif
+#endif
+
 #ifdef WIN32SERVICE
     trapd_status = SNMPTRAPD_RUNNING;
 #endif
@@ -1228,7 +1389,9 @@ main(int argc, char *argv[])
             trapd_update_config();
             if (trap1_fmt_str_remember) {
                 free_trap1_fmt();
+                free_trap2_fmt();
                 print_format1 = strdup(trap1_fmt_str_remember);
+                print_format2 = strdup(trap1_fmt_str_remember);
             }
             reconfig = 0;
         }
