@@ -238,9 +238,8 @@ typedef enum {
       */
 
 #define is_auth_cmd(chr) ((((chr) == CHR_SNMP_VERSION       \
-                            || (chr) == CHR_SNMP_VERSION     \
-                            || (chr) == CHR_SNMP_USER                   \
-                            || (chr) == CHR_TRAP_CONTEXTID)) ? TRUE : FALSE)
+                            || (chr) == CHR_SNMP_SECMOD     \
+                            || (chr) == CHR_SNMP_USER)) ? TRUE : FALSE)
 
      /*
       * Function:
@@ -270,6 +269,7 @@ typedef enum {
 			  || is_agent_cmd (chr)     \
 			  || is_pdu_ip_cmd (chr)    \
                           || ((chr) == CHR_PDU_ENT) \
+                          || ((chr) == CHR_TRAP_CONTEXTID) \
                           || ((chr) == CHR_PDU_WRAP) \
 			  || is_trap_cmd (chr)) ? TRUE : FALSE)
      /*
@@ -640,7 +640,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
          * Write a numerical address.  
          */
         if (!snmp_strcat(&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                         inet_ntoa(*agent_inaddr))) {
+                         (u_char *)inet_ntoa(*agent_inaddr))) {
             if (temp_buf != NULL) {
                 free(temp_buf);
             }
@@ -659,7 +659,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
         }
         if (host != NULL) {
             if (!snmp_strcat(&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                             host->h_name)) {
+                             (u_char *)host->h_name)) {
                 if (temp_buf != NULL) {
                     free(temp_buf);
                 }
@@ -667,7 +667,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
             }
         } else {
             if (!snmp_strcat(&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                             inet_ntoa(*agent_inaddr))) {
+                             (u_char *)inet_ntoa(*agent_inaddr))) {
                 if (temp_buf != NULL) {
                     free(temp_buf);
                 }
@@ -685,7 +685,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
                 transport->f_fmtaddr(transport, pdu->transport_data,
                                      pdu->transport_data_length);
             if (!snmp_strcat
-                (&temp_buf, &temp_buf_len, &temp_out_len, 1, tstr)) {
+                (&temp_buf, &temp_buf_len, &temp_out_len, 1, (u_char *)tstr)) {
                 if (tstr != NULL) {
                     free(tstr);
                 }
@@ -700,7 +700,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
         } else {
             if (!snmp_strcat
                 (&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                 "<UNKNOWN>")) {
+                 (const u_char*)"<UNKNOWN>")) {
                 if (temp_buf != NULL) {
                     free(temp_buf);
                 }
@@ -718,7 +718,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
          * for the UDP and TCP transport domains (we don't want to try to be
          * too clever here).  
          */
-#ifdef SNMP_TRANSPORT_TCP_DOMAIN
+#ifdef NETSNMP_TRANSPORT_TCP_DOMAIN
         if (transport != NULL && (transport->domain == netsnmpUDPDomain ||
                                   transport->domain ==
                                   netsnmp_snmpTCPDomain)) {
@@ -730,21 +730,26 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
              * transport_data but never mind -- the alternative is a lot of
              * munging strings from f_fmtaddr.  
              */
-            struct sockaddr_in *addr =
-                (struct sockaddr_in *) pdu->transport_data;
+typedef struct netsnmp_udp_addr_pair_s {   /* From snmpUDPDomain.c */
+    struct sockaddr_in remote_addr;
+    struct in_addr local_addr;
+} netsnmp_udp_addr_pair;
+
+            netsnmp_udp_addr_pair *addr =
+                (netsnmp_udp_addr_pair *) pdu->transport_data;
             if (addr != NULL
                 && pdu->transport_data_length ==
-                sizeof(struct sockaddr_in)) {
+                sizeof(netsnmp_udp_addr_pair)) {
                 if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
                                             NETSNMP_DS_APP_NUMERIC_IP)) {
                     host =
-                        gethostbyaddr((char *) &(addr->sin_addr),
+                        gethostbyaddr((char *) &(addr->remote_addr.sin_addr),
                                       sizeof(struct in_addr), AF_INET);
                 }
                 if (host != NULL) {
                     if (!snmp_strcat
                         (&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                         host->h_name)) {
+                         (u_char *)host->h_name)) {
                         if (temp_buf != NULL) {
                             free(temp_buf);
                         }
@@ -753,7 +758,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
                 } else {
                     if (!snmp_strcat
                         (&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                         inet_ntoa(addr->sin_addr))) {
+                         (u_char *)inet_ntoa(addr->remote_addr.sin_addr))) {
                         if (temp_buf != NULL) {
                             free(temp_buf);
                         }
@@ -763,7 +768,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
             } else {
                 if (!snmp_strcat
                     (&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                     "<UNKNOWN>")) {
+                     (const u_char*)"<UNKNOWN>")) {
                     if (temp_buf != NULL) {
                         free(temp_buf);
                     }
@@ -779,7 +784,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
                 transport->f_fmtaddr(transport, pdu->transport_data,
                                      pdu->transport_data_length);
             if (!snmp_strcat
-                (&temp_buf, &temp_buf_len, &temp_out_len, 1, tstr)) {
+                (&temp_buf, &temp_buf_len, &temp_out_len, 1, (u_char *)tstr)) {
                 if (tstr != NULL) {
                     free(tstr);
                 }
@@ -796,7 +801,7 @@ realloc_handle_ip_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
              * We are kind of stuck!  
              */
             if (!snmp_strcat(&temp_buf, &temp_buf_len, &temp_out_len, 1,
-                             "<UNKNOWN>")) {
+                             (const u_char*)"<UNKNOWN>")) {
                 if (temp_buf != NULL) {
                     free(temp_buf);
                 }
@@ -827,8 +832,8 @@ realloc_handle_ent_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
 
      /*
       * Function:
-      *     Handle a format command that deals with the enterprise 
-      * string.  Append the information to the buffer subject to the
+      *     Handle a format command that deals with OID strings. 
+      * Append the information to the buffer subject to the
       * buffer's length limit.
       *
       * Input Parameters:
@@ -862,6 +867,18 @@ realloc_handle_ent_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
         }
         break;
 
+    case CHR_TRAP_CONTEXTID:
+        /*
+         * Write the context oid.  
+         */
+        if (!sprint_realloc_objid
+            (&temp_buf, &temp_buf_len, &temp_out_len, 1, pdu->contextEngineID,
+             pdu->contextEngineIDLen)) {
+            free(temp_buf);
+            return 0;
+        }
+        break;
+
         /*
          * Don't know how to handle this command - write the character itself.  
          */
@@ -915,7 +932,7 @@ realloc_handle_trap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
         /*
          * Write the trap's number.  
          */
-        tout_len = sprintf(temp_buf, "%ld", pdu->trap_type);
+        tout_len = sprintf((char*)temp_buf, "%ld", pdu->trap_type);
         break;
 
     case CHR_TRAP_DESC:
@@ -923,7 +940,7 @@ realloc_handle_trap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
          * Write the trap's description.  
          */
         tout_len =
-            sprintf(temp_buf, "%s", trap_description(pdu->trap_type));
+            sprintf((char*)temp_buf, "%s", trap_description(pdu->trap_type));
         break;
 
     case CHR_TRAP_STYPE:
@@ -931,7 +948,7 @@ realloc_handle_trap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
          * Write the trap's subtype.  
          */
         if (pdu->trap_type != SNMP_TRAP_ENTERPRISESPECIFIC) {
-            tout_len = sprintf(temp_buf, "%ld", pdu->specific_type);
+            tout_len = sprintf((char*)temp_buf, "%ld", pdu->specific_type);
         } else {
             /*
              * Get object ID for the trap.  
@@ -1001,7 +1018,7 @@ realloc_handle_trap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
              */
             if (options->alt_format ||
                 vars != pdu->variables ) {
-                if (!snmp_strcat(&temp_buf, &tbuf_len, &tout_len, 1, sep)) {
+                if (!snmp_strcat(&temp_buf, &tbuf_len, &tout_len, 1, (const u_char *)sep)) {
                     if (temp_buf != NULL) {
                         free(temp_buf);
                     }
@@ -1051,23 +1068,45 @@ realloc_handle_auth_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
       *    pdu     - information about this trap 
       */
 {
-    netsnmp_variable_list *vars;        /* variables assoc with trap */
     char            fmt_cmd = options->cmd;     /* what we're outputting */
     u_char         *temp_buf = NULL;
     size_t          tbuf_len = 64, tout_len = 0;
-    const char           *sep = separator;
-    const char           *default_sep = "\t";
-    const char           *default_alt_sep = ", ";
+    int             i;
 
     if ((temp_buf = calloc(tbuf_len, 1)) == NULL) {
         return 0;
     }
 
-    fprintf(stderr, "here\n\n");
-    switch (pdu->command) {
+    switch (fmt_cmd) {
 
     case CHR_SNMP_VERSION:
-        *out_len = snprintf(temp_buf, tbuf_len, "%ld", pdu->version);
+        tout_len = snprintf((char*)temp_buf, tbuf_len, "%ld", pdu->version);
+        break;
+
+    case CHR_SNMP_SECMOD:
+        tout_len = snprintf((char*)temp_buf, tbuf_len, "%d", pdu->securityModel);
+        break;
+
+    case CHR_SNMP_USER:
+        if ( pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c ) {
+            while ((*out_len + pdu->community_len + 1) >= *buf_len) {
+                if (!(allow_realloc && snmp_realloc(buf, buf_len))) {
+                    return 0;
+                }
+            }
+
+            for (i = 0; i < pdu->community_len; i++) {
+                if (isprint(pdu->community[i])) {
+                    *(*buf + *out_len) = pdu->community[i];
+                } else {
+                    *(*buf + *out_len) = '.';
+                }
+                (*out_len)++;
+            }
+            *(*buf + *out_len) = '\0';
+        } else {
+            tout_len = snprintf((char*)temp_buf, tbuf_len, "%s", pdu->securityName);
+        }
         break;
 
     default:
@@ -1115,7 +1154,7 @@ realloc_handle_wrap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
     }
 
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
         if (!snmp_strcat
             (buf, buf_len, out_len, allow_realloc,
@@ -1124,7 +1163,7 @@ realloc_handle_wrap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
         }
         break;
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
         if (!snmp_strcat
             (buf, buf_len, out_len, allow_realloc,
@@ -1143,13 +1182,13 @@ realloc_handle_wrap_fmt(u_char ** buf, size_t * buf_len, size_t * out_len,
     }
 
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
 #endif
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
         if (!snmp_strcat
             (buf, buf_len, out_len, allow_realloc,
              (const u_char *) ", community ")) {
@@ -1258,7 +1297,7 @@ realloc_dispatch_format_cmd(u_char ** buf, size_t * buf_len,
     } else if (is_auth_cmd(fmt_cmd)) {
         return realloc_handle_auth_fmt(buf, buf_len, out_len,
                                        allow_realloc, options, pdu);
-    } else if (fmt_cmd == CHR_PDU_ENT) {
+    } else if (fmt_cmd == CHR_PDU_ENT || fmt_cmd == CHR_TRAP_CONTEXTID) {
         return realloc_handle_ent_fmt(buf, buf_len, out_len, allow_realloc,
                                       options, pdu);
     } else if (fmt_cmd == CHR_PDU_WRAP) {
@@ -1440,7 +1479,7 @@ realloc_format_plain_trap(u_char ** buf, size_t * buf_len,
             }
             return 0;
         }
-        if (!snmp_strcat(buf, buf_len, out_len, allow_realloc, tstr)) {
+        if (!snmp_strcat(buf, buf_len, out_len, allow_realloc, (u_char *)tstr)) {
             if (tstr != NULL) {
                 free(tstr);
             }
@@ -1482,7 +1521,7 @@ realloc_format_plain_trap(u_char ** buf, size_t * buf_len,
         return 0;
     }
     if (!snmp_strcat(buf, buf_len, out_len, allow_realloc,
-                     trap_description(pdu->trap_type))) {
+                     (const u_char *)trap_description(pdu->trap_type))) {
         return 0;
     }
     if (!snmp_strcat
