@@ -1,9 +1,19 @@
 #include "sysv.h"
 
+#undef hpux
+#define hpux hpux
+
 #ifdef hpux11
-#define DONT_USE_NLIST 1
+#define NETSNMP_DONT_USE_NLIST 1
 #endif
 
+/* 
+ * HP-UX needs _REENTRANT defined to pick up strtok_r.
+ * Otherwise, at least for 64-bit code, strtok_r will not work 
+ * and will make net-snmp segfault.
+ */
+#define _REENTRANT 1
+
 #undef TCP_TTL_SYMBOL
 #ifndef hpux11
 #define TCP_TTL_SYMBOL "ipDefaultTTL"
@@ -62,8 +72,6 @@
 
 #define rt_pad1 rt_refcnt
 
-#define hpux 1
-
 /*
  * disable inline for non-gcc compiler
  */
@@ -71,3 +79,6 @@
 #  undef NETSNMP_ENABLE_INLINE
 #  define NETSNMP_ENABLE_INLINE 0
 #endif
+
+/* define the extra mib modules that are supported */
+#define NETSNMP_INCLUDE_HOST_RESOURCES
