@@ -1,7 +1,7 @@
 /*
  * netsnmp_data_list.c
  *
- * $Id: data_list.c,v 5.11.2.1 2006/01/25 16:27:41 dts12 Exp $
+ * $Id: data_list.c 16612 2007-07-16 23:59:44Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -105,7 +105,7 @@ netsnmp_data_list_add_node(netsnmp_data_list **head, netsnmp_data_list *node)
 
     DEBUGMSGTL(("data_list","adding key '%s'\n", node->name));
     if (0 == strcmp(node->name, (*head)->name)) {
-        netsnmp_assert("list key" == "is unique"); /* always fail */
+        netsnmp_assert(!"list key == is unique"); /* always fail */
         snmp_log(LOG_WARNING,
                  "WARNING: adding duplicate key '%s' to data list\n",
                  node->name);
@@ -114,7 +114,7 @@ netsnmp_data_list_add_node(netsnmp_data_list **head, netsnmp_data_list *node)
     for (ptr = *head; ptr->next != NULL; ptr = ptr->next) {
         netsnmp_assert(NULL != ptr->name);
         if (0 == strcmp(node->name, ptr->name)) {
-            netsnmp_assert("list key" == "is unique"); /* always fail */
+            netsnmp_assert(!"list key == is unique"); /* always fail */
             snmp_log(LOG_WARNING,
                      "WARNING: adding duplicate key '%s' to data list\n",
                      node->name);
@@ -279,7 +279,7 @@ netsnmp_register_save_list(netsnmp_data_list **datalist,
 int
 netsnmp_save_all_data_callback(int major, int minor,
                                void *serverarg, void *clientarg) {
-    netsnmp_data_list_saveinfo *info = clientarg;
+    netsnmp_data_list_saveinfo *info = (netsnmp_data_list_saveinfo *)clientarg;
 
     if (!clientarg) {
         snmp_log(LOG_WARNING, "netsnmp_save_all_data_callback called with no passed data");
@@ -336,7 +336,7 @@ netsnmp_read_data_callback(const char *token, char *line) {
     void *data = NULL;
 
     /* find the stashed information about what we're parsing */
-    info = netsnmp_get_list_data(saveHead, token);
+    info = (netsnmp_data_list_saveinfo *) netsnmp_get_list_data(saveHead, token);
     if (!info) {
         snmp_log(LOG_WARNING, "netsnmp_read_data_callback called without previously registered subparser");
         return;
