@@ -139,72 +139,18 @@ static int      config_errors;
 
 struct config_files *config_files = NULL;
 
-struct config_line *
-register_prenetsnmp_mib_handler(const char *type,
-                                const char *token,
-                                void (*parser) (const char *, char *),
-                                void (*releaser) (void), const char *help)
-{
-    struct config_line *ltmp;
-    ltmp = register_config_handler(type, token, parser, releaser, help);
-    if (ltmp != NULL)
-        ltmp->config_time = PREMIB_CONFIG;
-    return (ltmp);
-}
-
-struct config_line *
-register_app_prenetsnmp_mib_handler(const char *token,
-                                    void (*parser) (const char *, char *),
-                                    void (*releaser) (void),
-                                    const char *help)
-{
-    return (register_prenetsnmp_mib_handler
-            (NULL, token, parser, releaser, help));
-}
-
-/**
- * register_config_handler registers handlers for certain tokens specified in
- * certain types of files.
- *
- * Allows a module writer use/register multiple configuration files based off
- * of the type parameter.  A module writer may want to set up multiple
- * configuration files to separate out related tasks/variables or just for
- * management of where to put tokens as the module or modules get more complex
- * in regard to handling token registrations.
- *
- * @param type_param the configuration file used, e.g., if snmp.conf is the file
- *                 where the token is located use "snmp" here.
- *                 If NULL the configuration file used will be snmpd.conf.
- *
- * @param token    the token being parsed from the file.  Must be non-NULL.
- *
- * @param parser   the handler function pointer that use  the specified
- *                 token and the rest of the line to do whatever is required
- *                 Should be non-NULL in order to make use of this API.
- *
- * @param releaser if non-NULL, the function specified is called if
- *                 and when the configuration files are re-read.  This function
- *                 should free any resources allocated by the token handler
- *                 function.
- *
- * @param help     if non-NULL, used to display help information on the expected 
- *	           arguments after the token.
- *      
- * @return Pointer to a new config line entry or NULL on error.
- */
-struct config_line *
-register_config_handler(const char *type_param,
-                        const char *token,
-                        void (*parser) (const char *, char *),
-                        void (*releaser) (void), const char *help)
+static struct config_line *
+internal_register_config_handler(const char *type_param,
+				 const char *token,
+				 void (*parser) (const char *, char *),
+				 void (*releaser) (void), const char *help,
+				 int when)
 {
     struct config_files **ctmp = &config_files;
-    struct config_line **ltmp, *ltmp2;
-    const char     *type = type_param;
-    char           *cptr, buf[STRINGMAX];
-    char           *st;
+    struct config_line  **ltmp;
+    const char           *type = type_param;
 
-    if (type == NULL) {
+    if (type == NULL || *type == '\0') {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				     NETSNMP_DS_LIB_APPTYPE);
     }
@@ -212,15 +158,20 @@ register_config_handler(const char *type_param,
     /*
      * Handle multiple types (recursively)
      */
-    cptr = strchr( type, ':' );
-    if (cptr) {
-        strncpy(buf, type, STRINGMAX);
+    if (strchr(type, ':')) {
+        struct config_line *ltmp2 = NULL;
+        char                buf[STRINGMAX];
+        char               *cptr = buf;
+        strncpy(buf, type, STRINGMAX - 1);
         buf[STRINGMAX - 1] = '\0';
-        ltmp2 = NULL;
-        cptr = strtok_r(buf, ":", &st);
         while (cptr) {
-            ltmp2 = register_config_handler(cptr, token, parser, releaser, help);
-            cptr  = strtok_r(NULL, ":", &st);
+            char* c = cptr;
+            cptr = strchr(cptr, ':');
+            if(cptr) {
+                *cptr = '\0';
+                ++cptr;
+            }
+            ltmp2 = register_config_handler(c, token, parser, releaser, help);
         }
         return ltmp2;
     }
@@ -259,7 +210,7 @@ register_config_handler(const char *type_param,
             return NULL;
         }
 
-        (*ltmp)->config_time = NORMAL_CONFIG;
+        (*ltmp)->config_time = when;
         (*ltmp)->config_token = strdup(token);
         if (help != NULL)
             (*ltmp)->help = strdup(help);
@@ -276,6 +227,68 @@ register_config_handler(const char *type_param,
 
 }                               /* end register_config_handler() */
 
+struct config_line *
+register_prenetsnmp_mib_handler(const char *type,
+                                const char *token,
+                                void (*parser) (const char *, char *),
+                                void (*releaser) (void), const char *help)
+{
+    return internal_register_config_handler(type, token, parser, releaser,
+					    help, PREMIB_CONFIG);
+}
+
+struct config_line *
+register_app_prenetsnmp_mib_handler(const char *token,
+                                    void (*parser) (const char *, char *),
+                                    void (*releaser) (void),
+                                    const char *help)
+{
+    return (register_prenetsnmp_mib_handler
+            (NULL, token, parser, releaser, help));
+}
+
+/**
+ * register_config_handler registers handlers for certain tokens specified in
+ * certain types of files.
+ *
+ * Allows a module writer use/register multiple configuration files based off
+ * of the type parameter.  A module writer may want to set up multiple
+ * configuration files to separate out related tasks/variables or just for
+ * management of where to put tokens as the module or modules get more complex
+ * in regard to handling token registrations.
+ *
+ * @param type_param the configuration file used, e.g., if snmp.conf is the
+ *                 file where the token is located use "snmp" here.
+ *                 Multiple colon separated tokens might be used.
+ *                 If NULL or "" then the configuration file used will be
+ *                 <application>.conf.
+ *
+ * @param token    the token being parsed from the file.  Must be non-NULL.
+ *
+ * @param parser   the handler function pointer that use  the specified
+ *                 token and the rest of the line to do whatever is required
+ *                 Should be non-NULL in order to make use of this API.
+ *
+ * @param releaser if non-NULL, the function specified is called if
+ *                 and when the configuration files are re-read.  This function
+ *                 should free any resources allocated by the token handler
+ *                 function.
+ *
+ * @param help     if non-NULL, used to display help information on the
+ *                 expected arguments after the token.
+ *
+ * @return Pointer to a new config line entry or NULL on error.
+ */
+struct config_line *
+register_config_handler(const char *type,
+			const char *token,
+			void (*parser) (const char *, char *),
+			void (*releaser) (void), const char *help)
+{
+    return internal_register_config_handler(type, token, parser, releaser,
+					    help, NORMAL_CONFIG);
+}
+
 struct config_line *
 register_app_config_handler(const char *token,
                             void (*parser) (const char *, char *),
@@ -301,12 +314,10 @@ void
 unregister_config_handler(const char *type_param, const char *token)
 {
     struct config_files **ctmp = &config_files;
-    struct config_line **ltmp, *ltmp2;
-    const char     *type = type_param;
-    char           *cptr, buf[STRINGMAX];
-    char           *st;
+    struct config_line  **ltmp;
+    const char           *type = type_param;
 
-    if (type == NULL) {
+    if (type == NULL || *type == '\0') {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				     NETSNMP_DS_LIB_APPTYPE);
     }
@@ -314,14 +325,19 @@ unregister_config_handler(const char *type_param, const char *token)
     /*
      * Handle multiple types (recursively)
      */
-    cptr = strchr( type, ':' );
-    if (cptr) {
-        strncpy(buf, type, STRINGMAX);
+    if (strchr(type, ':')) {
+        char                buf[STRINGMAX];
+        char               *cptr = buf;
+        strncpy(buf, type, STRINGMAX - 1);
         buf[STRINGMAX - 1] = '\0';
-        cptr = strtok_r(buf, ":", &st);
         while (cptr) {
-            unregister_config_handler(cptr, token);
-            cptr  = strtok_r(NULL, ":", &st);
+            char* c = cptr;
+            cptr = strchr(cptr, ':');
+            if(cptr) {
+                *cptr = '\0';
+                ++cptr;
+            }
+            unregister_config_handler(c, token);
         }
         return;
     }
@@ -351,7 +367,7 @@ unregister_config_handler(const char *type_param, const char *token)
         /*
          * found it at the top of the list 
          */
-        ltmp2 = (*ltmp)->next;
+        struct config_line *ltmp2 = (*ltmp)->next;
         SNMP_FREE((*ltmp)->config_token);
         SNMP_FREE((*ltmp)->help);
         SNMP_FREE(*ltmp);
@@ -363,9 +379,9 @@ unregister_config_handler(const char *type_param, const char *token)
         ltmp = &((*ltmp)->next);
     }
     if ((*ltmp)->next != NULL) {
+        struct config_line *ltmp2 = (*ltmp)->next->next;
         SNMP_FREE((*ltmp)->next->config_token);
         SNMP_FREE((*ltmp)->next->help);
-        ltmp2 = (*ltmp)->next->next;
         SNMP_FREE((*ltmp)->next);
         (*ltmp)->next = ltmp2;
     }
@@ -803,7 +819,7 @@ free_config(void)
 void
 read_configs_optional(const char *optional_config, int when)
 {
-    char *newp, *cp, *st;
+    char *newp, *cp, *st = NULL;
     char *type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				       NETSNMP_DS_LIB_APPTYPE);
 
@@ -967,9 +983,9 @@ get_persistent_directory()
 {
     if (NULL == netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				      NETSNMP_DS_LIB_PERSISTENT_DIR)) {
-        char *persdir = netsnmp_getenv("SNMP_PERSISTENT_DIR");
+        const char *persdir = netsnmp_getenv("SNMP_PERSISTENT_DIR");
         if (NULL == persdir)
-            persdir = PERSISTENT_DIRECTORY;
+            persdir = NETSNMP_PERSISTENT_DIRECTORY;
         set_persistent_directory(persdir);
     }
     return (netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
@@ -1034,13 +1050,34 @@ read_config_files_in_path(const char *path, struct config_files *ctmp,
                 ctmp->fileHeader, envconfpath, perspath));
     cptr1 = cptr2 = envconfpath;
     done = 0;
-    while (*cptr2 != 0) {
+    while ((*cptr2 != 0) && (!done)) {
         while (*cptr1 != 0 && *cptr1 != ENV_SEPARATOR_CHAR)
             cptr1++;
         if (*cptr1 == 0)
             done = 1;
         else
             *cptr1 = 0;
+
+        DEBUGMSGTL(("read_config", " config dir: %s\n", cptr2 ));
+        if (stat(cptr2, &statbuf) != 0) {
+            /*
+             * Directory not there, continue 
+             */
+            DEBUGMSGTL(("read_config", " Directory not present: %s\n", cptr2 ));
+            cptr2 = ++cptr1;
+            continue;
+        }
+#ifdef S_ISDIR
+        if (!S_ISDIR(statbuf.st_mode)) {
+            /*
+             * Not a directory, continue 
+             */
+            DEBUGMSGTL(("read_config", " Not a directory: %s\n", cptr2 ));
+            cptr2 = ++cptr1;
+            continue;
+        }
+#endif
+
         /*
          * for proper persistent storage retrival, we need to read old backup
          * copies of the previous storage files.  If the application in
@@ -1054,7 +1091,7 @@ read_config_files_in_path(const char *path, struct config_files *ctmp,
             /*
              * limit this to the known storage directory only 
              */
-            for (j = 0; j <= MAX_PERSISTENT_BACKUPS; j++) {
+            for (j = 0; j <= NETSNMP_MAX_PERSISTENT_BACKUPS; j++) {
                 snprintf(configfile, sizeof(configfile),
                          "%s/%s.%d.conf", cptr2,
                          ctmp->fileHeader, j);
@@ -1216,7 +1253,7 @@ read_config_print_usage(const char *lead)
  * read_config_store intended for use by applications to store permenant
  * configuration information generated by sets or persistent counters.
  * Appends line to a file named either ENV(SNMP_PERSISTENT_FILE) or
- *   "<PERSISTENT_DIRECTORY>/<type>.conf".
+ *   "<NETSNMP_PERSISTENT_DIRECTORY>/<type>.conf".
  * Adds a trailing newline to the stored file if necessary.
  *
  * @param type is the application name
@@ -1228,10 +1265,10 @@ read_config_print_usage(const char *lead)
 void
 read_config_store(const char *type, const char *line)
 {
-#ifdef PERSISTENT_DIRECTORY
+#ifdef NETSNMP_PERSISTENT_DIRECTORY
     char            file[512], *filep;
     FILE           *fout;
-#ifdef PERSISTENT_MASK
+#ifdef NETSNMP_PERSISTENT_MASK
     mode_t          oldmask;
 #endif
 
@@ -1243,7 +1280,7 @@ read_config_store(const char *type, const char *line)
     /*
      * store configuration directives in the following order of preference:
      * 1. ENV variable SNMP_PERSISTENT_FILE
-     * 2. configured <PERSISTENT_DIRECTORY>/<type>.conf
+     * 2. configured <NETSNMP_PERSISTENT_DIRECTORY>/<type>.conf
      */
     if ((filep = netsnmp_getenv("SNMP_PERSISTENT_FILE")) == NULL) {
         snprintf(file, sizeof(file),
@@ -1251,10 +1288,10 @@ read_config_store(const char *type, const char *line)
         file[ sizeof(file)-1 ] = 0;
         filep = file;
     }
-#ifdef PERSISTENT_MASK
-    oldmask = umask(PERSISTENT_MASK);
+#ifdef NETSNMP_PERSISTENT_MASK
+    oldmask = umask(NETSNMP_PERSISTENT_MASK);
 #endif
-    if (mkdirhier(filep, AGENT_DIRECTORY_MODE, 1)) {
+    if (mkdirhier(filep, NETSNMP_AGENT_DIRECTORY_MODE, 1)) {
         snmp_log(LOG_ERR,
                  "Failed to create the persistent directory for %s\n",
                  file);
@@ -1268,7 +1305,7 @@ read_config_store(const char *type, const char *line)
     } else {
         snmp_log(LOG_ERR, "read_config_store open failure on %s\n", filep);
     }
-#ifdef PERSISTENT_MASK
+#ifdef NETSNMP_PERSISTENT_MASK
     umask(oldmask);
 #endif
 
@@ -1292,9 +1329,9 @@ read_app_config_store(const char *line)
  *	*type
  *      
  *
- * Save the file "<PERSISTENT_DIRECTORY>/<type>.conf" into a backup copy
- * called "<PERSISTENT_DIRECTORY>/<type>.%d.conf", which %d is an
- * incrementing number on each call, but less than MAX_PERSISTENT_BACKUPS.
+ * Save the file "<NETSNMP_PERSISTENT_DIRECTORY>/<type>.conf" into a backup copy
+ * called "<NETSNMP_PERSISTENT_DIRECTORY>/<type>.%d.conf", which %d is an
+ * incrementing number on each call, but less than NETSNMP_MAX_PERSISTENT_BACKUPS.
  *
  * Should be called just before all persistent information is supposed to be
  * written to move aside the existing persistent cache.
@@ -1321,7 +1358,7 @@ snmp_save_persistent(const char *type)
              "%s/%s.conf", get_persistent_directory(), type);
     file[ sizeof(file)-1 ] = 0;
     if (stat(file, &statbuf) == 0) {
-        for (j = 0; j <= MAX_PERSISTENT_BACKUPS; j++) {
+        for (j = 0; j <= NETSNMP_MAX_PERSISTENT_BACKUPS; j++) {
             snprintf(fileold, sizeof(fileold),
                      "%s/%s.%d.conf", get_persistent_directory(), type, j);
             fileold[ sizeof(fileold)-1 ] = 0;
@@ -1358,7 +1395,7 @@ snmp_save_persistent(const char *type)
  *	*type
  *      
  *
- * Unlink all backup files called "<PERSISTENT_DIRECTORY>/<type>.%d.conf".
+ * Unlink all backup files called "<NETSNMP_PERSISTENT_DIRECTORY>/<type>.%d.conf".
  *
  * Should be called just after we successfull dumped the last of the
  * persistent data, to remove the backup copies of previous storage dumps.
@@ -1385,7 +1422,7 @@ snmp_clean_persistent(const char *type)
              "%s/%s.conf", get_persistent_directory(), type);
     file[ sizeof(file)-1 ] = 0;
     if (stat(file, &statbuf) == 0) {
-        for (j = 0; j <= MAX_PERSISTENT_BACKUPS; j++) {
+        for (j = 0; j <= NETSNMP_MAX_PERSISTENT_BACKUPS; j++) {
             snprintf(file, sizeof(file),
                      "%s/%s.%d.conf", get_persistent_directory(), type, j);
             file[ sizeof(file)-1 ] = 0;
@@ -1641,9 +1678,9 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
                 snmp_log(LOG_WARNING,"buffer too small to read octet string (%d < %d)\n",
                          *len, ilen);
                 DEBUGMSGTL(("read_config_read_octet_string",
-                            "buffer too small (%d < %d)", *len, ilen));
-                cptr = skip_not_white(readfrom);
-                return skip_white(cptr);
+                            "buffer too small (%lu < %lu)", (unsigned long)*len, (unsigned long)ilen));
+                cptr1 = skip_not_white(readfrom);
+                return skip_white(cptr1);
             }
             cptr = *str;
         }
@@ -1692,7 +1729,7 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
             }
         } else {
             readfrom = copy_nword(readfrom, (char *) *str, *len);
-            *len = strlen(*str);
+            *len = strlen((char *) *str);
         }
     }
 
@@ -1813,6 +1850,15 @@ read_config_read_data(int type, char *readfrom, void *dataptr,
             readfrom = skip_token(readfrom);
             return readfrom;
 
+        case ASN_IPADDRESS:
+            intp = (int *) dataptr;
+            *intp = inet_addr(readfrom);
+            if ((*intp == -1) &&
+                (strncmp(readfrom, "255.255.255.255", 15) != 0))
+                return NULL;
+            readfrom = skip_token(readfrom);
+            return readfrom;
+
         case ASN_OCTET_STR:
         case ASN_BIT_STR:
             charpp = (char **) dataptr;
@@ -1870,6 +1916,17 @@ read_config_read_memory(int type, char *readfrom,
         *len = sizeof(unsigned int);
         return readfrom;
 
+    case ASN_IPADDRESS:
+        if (*len < sizeof(int))
+            return NULL;
+        intp = (int *) dataptr;
+        readfrom = copy_nword(readfrom, buf, sizeof(buf));
+        *intp = inet_addr(buf);
+        if ((*intp == -1) && (strcmp(buf, "255.255.255.255") != 0))
+            return NULL;
+        *len = sizeof(int);
+        return readfrom;
+
     case ASN_OCTET_STR:
     case ASN_BIT_STR:
     case ASN_PRIV_IMPLIED_OCTET_STR:
@@ -1942,6 +1999,7 @@ read_config_store_data_prefix(char prefix, int type, char *storeto,
     int            *intp;
     u_char        **charpp;
     unsigned int   *uintp;
+    struct in_addr  in;
     oid           **oidpp;
 
     if (dataptr && storeto)
@@ -1957,6 +2015,11 @@ read_config_store_data_prefix(char prefix, int type, char *storeto,
             sprintf(storeto, "%c%u", prefix, *uintp);
             return (storeto + strlen(storeto));
 
+        case ASN_IPADDRESS:
+            in.s_addr = *(unsigned int *) dataptr; 
+            sprintf(storeto, "%c%s", prefix, inet_ntoa(in));
+            return (storeto + strlen(storeto));
+
         case ASN_OCTET_STR:
         case ASN_BIT_STR:
             *storeto++ = prefix;
