@@ -33,7 +33,7 @@
 #include <net-snmp/library/snmpAAL5PVCDomain.h>
 
 
-oid netsnmp_AAL5PVCDomain[10] = { ENTERPRISE_MIB, 3, 3, 3 };
+oid netsnmp_AAL5PVCDomain[10] = { NETSNMP_ENTERPRISE_MIB, 3, 3, 3 };
 static netsnmp_tdomain aal5pvcDomain;
 
 
@@ -216,7 +216,7 @@ netsnmp_aal5pvc_transport(struct sockaddr_atmpvc *addr, int local)
     }
 
     if (local) {
-        t->local = malloc(8);
+        t->local = (unsigned char*)malloc(8);
         if (t->local == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -240,7 +240,7 @@ netsnmp_aal5pvc_transport(struct sockaddr_atmpvc *addr, int local)
             return NULL;
         }
     } else {
-        t->remote = malloc(8);
+        t->remote = (unsigned char*)malloc(8);
         if (t->remote == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -290,10 +290,14 @@ netsnmp_aal5pvc_transport(struct sockaddr_atmpvc *addr, int local)
 
 
 netsnmp_transport *
-netsnmp_aal5pvc_create_tstring(const char *str, int local)
+netsnmp_aal5pvc_create_tstring(const char *str, int local,
+			       const char *default_target)
 {
     struct sockaddr_atmpvc addr;
 
+    if (str == NULL || *str == '\0')
+	str = default_target;
+
     if (str != NULL) {
         addr.sap_family = AF_ATMPVC;
 
@@ -342,11 +346,11 @@ netsnmp_aal5pvc_ctor(void)
 {
     aal5pvcDomain.name = netsnmp_AAL5PVCDomain;
     aal5pvcDomain.name_length = sizeof(netsnmp_AAL5PVCDomain) / sizeof(oid);
-    aal5pvcDomain.prefix = calloc(3, sizeof(char *));
+    aal5pvcDomain.prefix = (const char**)calloc(3, sizeof(char *));
     aal5pvcDomain.prefix[0] = "aal5pvc";
     aal5pvcDomain.prefix[1] = "pvc";
 
-    aal5pvcDomain.f_create_from_tstring = netsnmp_aal5pvc_create_tstring;
+    aal5pvcDomain.f_create_from_tstring_new = netsnmp_aal5pvc_create_tstring;
     aal5pvcDomain.f_create_from_ostring = netsnmp_aal5pvc_create_ostring;
 
     netsnmp_tdomain_register(&aal5pvcDomain);
