@@ -1,6 +1,6 @@
 #include <net-snmp/net-snmp-config.h>
 
-#ifdef SNMP_TRANSPORT_UDPIPV6_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
 
 /*
  * hack-o-matic for Cygwin to use winsock2
@@ -67,6 +67,12 @@ const struct in6_addr in6addr_any = IN6ADDR_ANY_INIT;
 #include <dmalloc.h>
 #endif
 
+#if STRUCT_SOCKADDR_STORAGE_HAS_SS_FAMILY
+#define SS_FAMILY ss_family
+#elif STRUCT_SOCKADDR_STORAGE_HAS___SS_FAMILY
+#define SS_FAMILY __ss_family
+#endif
+
 #include <net-snmp/types.h>
 #include <net-snmp/output_api.h>
 #include <net-snmp/config_api.h>
@@ -106,7 +112,7 @@ netsnmp_udp6_fmtaddr(netsnmp_transport *t, void *data, int len)
         char addr[INET6_ADDRSTRLEN];
         char tmp[INET6_ADDRSTRLEN + 8];
 
-        sprintf(tmp, "UDP/IPv6: [%s]:%hd",
+        sprintf(tmp, "UDP/IPv6: [%s]:%hu",
                 inet_ntop(AF_INET6, (void *) &(to->sin6_addr), addr,
                           INET6_ADDRSTRLEN), ntohs(to->sin6_port));
         return strdup(tmp);
@@ -281,7 +287,7 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
             netsnmp_transport_free(t);
             return NULL;
         }
-        t->local = malloc(18);
+        t->local = (unsigned char*)malloc(18);
         if (t->local == NULL) {
             netsnmp_udp6_close(t);
             netsnmp_transport_free(t);
@@ -307,7 +313,7 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
         }
         memcpy(t->data, addr, sizeof(struct sockaddr_in6));
         t->data_length = sizeof(struct sockaddr_in6);
-        t->remote = malloc(18);
+        t->remote = (unsigned char*)malloc(18);
         if (t->remote == NULL) {
             netsnmp_udp6_close(t);
             netsnmp_transport_free(t);
@@ -334,10 +340,12 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
 }
 
 
-
+/*
+ * Not extern but used from here and snmpTCPIPv6Domain.C
+ */
 int
-netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
-                     const char *inpeername, int remote_port)
+netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
+                       const char *inpeername, const char *default_target)
 {
     char           *cp = NULL, *peername = NULL;
     char            debug_addr[INET6_ADDRSTRLEN];
@@ -351,26 +359,29 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
 #elif HAVE_GETHOSTBYNAME
     struct hostent *hp = NULL;
 #endif
+    int             portno;
 
     if (addr == NULL) {
         return 0;
     }
 
-    DEBUGMSGTL(("netsnmp_sockaddr_in6", "addr %p, peername \"%s\"\n",
-                addr, inpeername ? inpeername : "[NIL]"));
+    DEBUGMSGTL(("netsnmp_sockaddr_in6",
+		"addr %p, peername \"%s\", default_target \"%s\"\n",
+                addr, inpeername ? inpeername : "[NIL]",
+		default_target ? default_target : "[NIL]"));
 
     memset(addr, 0, sizeof(struct sockaddr_in6));
     addr->sin6_family = AF_INET6;
     addr->sin6_addr = in6addr_any;
+    addr->sin6_port = htons((u_short)SNMP_PORT);
 
-    if (remote_port > 0) {
-        addr->sin6_port = htons(remote_port);
-    } else if (netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
-				  NETSNMP_DS_LIB_DEFAULT_PORT) > 0) {
-        addr->sin6_port = htons(netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
-						 NETSNMP_DS_LIB_DEFAULT_PORT));
-    } else {
-        addr->sin6_port = htons(SNMP_PORT);
+    {
+      int port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+				    NETSNMP_DS_LIB_DEFAULT_PORT);
+      if (port != 0)
+        addr->sin6_port = htons((u_short)port);
+      else if (default_target != NULL)
+	netsnmp_sockaddr_in6_2(addr, default_target, NULL);
     }
 
     if (inpeername != NULL) {
@@ -429,7 +440,7 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
                                     "IPv6 address with port suffix :%d\n",
                                     atoi(cp + 2)));
                         addr->sin6_port = htons(atoi(cp + 2));
-#ifdef HAVE_IF_NAMETOINDEX
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                         addr->sin6_scope_id = if_index;
 #endif
                         goto resolved;
@@ -440,8 +451,12 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
                          (void *) &(addr->sin6_addr))) {
                         DEBUGMSGTL(("netsnmp_sockaddr_in6",
                                     "IPv6 address with square brankets\n"));
-                        addr->sin6_port = htons(SNMP_PORT);
-#ifdef HAVE_IF_NAMETOINDEX
+                        portno = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+				                    NETSNMP_DS_LIB_DEFAULT_PORT);
+                        if (portno <= 0)
+                            portno = SNMP_PORT;
+                        addr->sin6_port = htons(portno);
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                         addr->sin6_scope_id = if_index;
 #endif
                         goto resolved;
@@ -475,7 +490,7 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
                             "IPv6 address with port suffix :%d\n",
                             atoi(cp + 1)));
                 addr->sin6_port = htons(atoi(cp + 1));
-#ifdef HAVE_IF_NAMETOINDEX
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                 addr->sin6_scope_id = if_index;
 #endif
                 goto resolved;
@@ -529,8 +544,12 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
 
         err = getaddrinfo(peername, NULL, &hint, &addrs);
         if (err != 0) {
+#if HAVE_GAI_STRERROR
             snmp_log(LOG_ERR, "getaddrinfo: %s %s\n", peername,
                      gai_strerror(err));
+#else
+            snmp_log(LOG_ERR, "getaddrinfo: %s (error %d)\n", peername, err);
+#endif
             free(peername);
             return 0;
         }
@@ -596,6 +615,15 @@ netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
 }
 
 
+int
+netsnmp_sockaddr_in6(struct sockaddr_in6 *addr,
+                     const char *inpeername, int remote_port)
+{
+    char buf[sizeof(remote_port) * 3 + 2];
+    sprintf(buf, ":%u", remote_port);
+    return netsnmp_sockaddr_in6_2(addr, inpeername, remote_port ? buf : NULL);
+}
+
 /*
  * int
  * inet_make_mask_addr( int pf, void *dst, int masklength )
@@ -868,10 +896,10 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
 
     switch (af) {
     case PF_INET:
-        if (mask->ss_family != PF_INET || network->ss_family != PF_INET) {
+        if (mask->SS_FAMILY != PF_INET || network->SS_FAMILY != PF_INET) {
             return -1;
         }
-        ss.ss_family = PF_INET;
+        ss.SS_FAMILY = PF_INET;
         inet_addr_and(PF_INET,
                       &((struct sockaddr_in *) from)->sin_addr,
                       &((struct sockaddr_in *) mask)->sin_addr,
@@ -884,10 +912,10 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
         }
         break;
     case PF_INET6:
-        if (mask->ss_family != PF_INET6 || network->ss_family != PF_INET6) {
+        if (mask->SS_FAMILY != PF_INET6 || network->SS_FAMILY != PF_INET6) {
             return -1;
         }
-        ss.ss_family = PF_INET6;
+        ss.SS_FAMILY = PF_INET6;
         inet_addr_and(PF_INET6,
                       &((struct sockaddr_in6 *) from)->sin6_addr,
                       &((struct sockaddr_in6 *) mask)->sin6_addr,
@@ -908,7 +936,7 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
     }
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /*
  * The following functions provide the "com2sec6" configuration token
  * functionality for compatibility.  
@@ -918,7 +946,7 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
 #define EXAMPLE_COMMUNITY     "COMMUNITY"
 
 typedef struct _com2Sec6Entry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     struct sockaddr_in6 network;
     struct sockaddr_in6 mask;
     char            secName[VACMSTRINGLEN];
@@ -954,7 +982,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
 {
     char            secName[VACMSTRINGLEN];
     char            contextName[VACMSTRINGLEN];
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            source[IPV6_STRING_LEN];
     char           *cp = NULL, *strnetwork = NULL, *strmask = NULL;
     com2Sec6Entry  *e = NULL;
@@ -1009,7 +1037,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
             == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
@@ -1201,7 +1229,7 @@ netsnmp_udp6_com2Sec6List_free(void)
 void
 netsnmp_udp6_agent_config_tokens_register(void)
 {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     register_app_config_handler("com2sec6", netsnmp_udp6_parse_security,
                                 netsnmp_udp6_com2Sec6List_free,
                                 "[-Cn CONTEXT] secName IPv6-network-address[/netmask] community");
@@ -1210,7 +1238,7 @@ netsnmp_udp6_agent_config_tokens_register(void)
 
 
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /*
  * Return 0 if there are no com2sec entries, or return 1 if there ARE com2sec 
  * entries.  On return, if a com2sec entry matched the passed parameters,
@@ -1293,11 +1321,12 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
 #endif /* support for community based SNMP */
 
 netsnmp_transport *
-netsnmp_udp6_create_tstring(const char *str, int local)
+netsnmp_udp6_create_tstring(const char *str, int local,
+			    const char *default_target)
 {
     struct sockaddr_in6 addr;
 
-    if (netsnmp_sockaddr_in6(&addr, str, 0)) {
+    if (netsnmp_sockaddr_in6_2(&addr, str, default_target)) {
         return netsnmp_udp6_transport(&addr, local);
     } else {
         return NULL;
@@ -1335,9 +1364,9 @@ netsnmp_udp6_ctor(void)
 {
     udp6Domain.name = netsnmp_UDPIPv6Domain;
     udp6Domain.name_length = sizeof(netsnmp_UDPIPv6Domain) / sizeof(oid);
-    udp6Domain.f_create_from_tstring = netsnmp_udp6_create_tstring;
+    udp6Domain.f_create_from_tstring_new = netsnmp_udp6_create_tstring;
     udp6Domain.f_create_from_ostring = netsnmp_udp6_create_ostring;
-    udp6Domain.prefix = calloc(5, sizeof(char *));
+    udp6Domain.prefix = (const char**)calloc(5, sizeof(char *));
     udp6Domain.prefix[0] = "udp6";
     udp6Domain.prefix[1] = "ipv6";
     udp6Domain.prefix[2] = "udpv6";
@@ -1355,5 +1384,5 @@ netsnmp_udp6_agent_config_tokens_register(void)
 { }
 #endif
 
-#endif /* SNMP_TRANSPORT_UDPIPV6_DOMAIN */
+#endif /* NETSNMP_TRANSPORT_UDPIPV6_DOMAIN */
 
