@@ -807,16 +807,10 @@ snmp_set_var_value(netsnmp_variable_list * vars,
                 }
             }
 #endif
-#if SIZEOF_LONG != SIZEOF_LONG_LONG
-#if defined (WIN32) && !defined (mingw32)
-            else if (vars->val_len == sizeof(__int64)){
-                const unsigned __int64   *val_ullong
-                    = (const unsigned __int64 *) value;
-#else
-                else if (vars->val_len == sizeof(long long)){
+#if defined(SIZEOF_LONG_LONG) && (SIZEOF_LONG != SIZEOF_LONG_LONG) && (SIZEOF_LONG_LONG != SIZEOF_INTMAX_T)
+            else if (vars->val_len == sizeof(long long)){
                 const unsigned long long   *val_ullong
                     = (const unsigned long long *) value;
-#endif
                 *(vars->val.integer) = (long) *val_ullong;
                 if (*(vars->val.integer) > 0xffffffff) {
                     snmp_log(LOG_ERR,"truncating integer value > 32 bits\n");
@@ -824,6 +818,17 @@ snmp_set_var_value(netsnmp_variable_list * vars,
                 }
             }
 #endif
+#if SIZEOF_LONG != SIZEOF_INTMAX_T
+            else if (vars->val_len == sizeof(intmax_t)){
+                const uintmax_t *val_uintmax_t
+                    = (const uintmax_t *) value;
+                *(vars->val.integer) = (long) *val_uintmax_t;
+                if (*(vars->val.integer) > 0xffffffff) {
+                    snmp_log(LOG_ERR,"truncating integer value > 32 bits\n");
+                    *(vars->val.integer) &= 0xffffffff;
+                }
+            }
+#endif
 #if SIZEOF_SHORT != SIZEOF_INT
             else if (vars->val_len == sizeof(short)) {
                 if (ASN_INTEGER == vars->type) {
@@ -884,7 +889,7 @@ snmp_set_var_value(netsnmp_variable_list * vars,
             vars->val.string = (u_char *) malloc(vars->val_len + 1);
         }
         if (vars->val.string == NULL) {
-            snmp_log(LOG_ERR,"no storage for OID\n");
+            snmp_log(LOG_ERR,"no storage for string\n");
             return 1;
         }
         memmove(vars->val.string, value, vars->val_len);
@@ -942,7 +947,7 @@ snmp_set_var_value(netsnmp_variable_list * vars,
 #endif                          /* NETSNMP_WITH_OPAQUE_SPECIAL_TYPES */
 
     default:
-        snmp_log(LOG_ERR,"no storage for OID\n");
+        snmp_log(LOG_ERR,"Internal error in type switching\n");
         snmp_set_detail("Internal error in type switching\n");
         return (1);
     }
@@ -1235,6 +1240,13 @@ retry:
                     break;
                 }
                 vtmp = vb2->next_variable;
+                /* free old data before overwriting */
+                if (vb2->val.string) {
+                    if (vb2->val.string != &vb2->buf[0]) {
+                        free(vb2->val.string);
+                        vb2->val.string = NULL;
+                    }
+                }
                 snmp_clone_var( vb1, vb2 );
                 vb2->next_variable = vtmp;
             }
