@@ -39,7 +39,7 @@
 #define SNMP_DEBUG_ACTIVE             1
 #define SNMP_DEBUG_EXCLUDED           2
 
-static int      dodebug = SNMP_ALWAYS_DEBUG;
+static int      dodebug = NETSNMP_ALWAYS_DEBUG;
 int             debug_num_tokens = 0;
 int             debug_num_excluded = 0;
 static int      debug_print_everything = 0;
@@ -136,7 +136,7 @@ void
 debug_register_tokens(char *tokens)
 {
     char           *newp, *cp;
-    char           *st;
+    char           *st = NULL;
     int             status;
 
     if (tokens == 0 || *tokens == 0)
@@ -299,7 +299,7 @@ debug_is_token_registered(const char *token)
             CONTAINER_INSERT(dbg_stats, found);
         }
         ++found->enabled;
-/*        snmp_log(LOG_ERR,"tok %s, %d hits\n", token, found->enabled); */
+    /*  snmp_log(LOG_ERR,"tok %s, %d hits\n", token, found->enabled);  */
     }
 #endif
 
@@ -423,11 +423,11 @@ debugmsg_oidrange(const char *token, const oid * theoid, size_t len,
         if (rc) {
             sprintf(tmpbuf, ".%lu--%lu", theoid[var_subid - 1],
                     range_ubound);
-            rc = snmp_strcat(&buf, &buf_len, &out_len, 1, tmpbuf);
+            rc = snmp_cstrcat(&buf, &buf_len, &out_len, 1, tmpbuf);
             if (rc) {
                 for (i = var_subid; i < len; i++) {
                     sprintf(tmpbuf, ".%lu", theoid[i]);
-                    if (!snmp_strcat(&buf, &buf_len, &out_len, 1, tmpbuf)) {
+                    if (!snmp_cstrcat(&buf, &buf_len, &out_len, 1, tmpbuf)) {
                         break;
                     }
                 }
@@ -581,7 +581,7 @@ _debug_cmp( const void *lhs, const void *rhs )
     netsnmp_token_descr *dbg_l = (netsnmp_token_descr *)lhs;
     netsnmp_token_descr *dbg_r = (netsnmp_token_descr *)rhs;
 
-/*    snmp_log(LOG_ERR,"%s/%s\n",dbg_l->token_name, dbg_r->token_name); */
+ /* snmp_log(LOG_ERR,"%s/%s\n",dbg_l->token_name, dbg_r->token_name); */
     return strcmp(dbg_l->token_name, dbg_r->token_name);
 }
 
