@@ -130,12 +130,21 @@ SOFTWARE.
 #include <sys/utsname.h>
 #endif
 
+#if HAVE_SYS_SYSTEMCFG_H
+#include <sys/systemcfg.h>
+#endif
+
+#if HAVE_SYS_SYSTEMINFO_H
+#include <sys/systeminfo.h>
+#endif
+
 #include <net-snmp/types.h>
 #include <net-snmp/output_api.h>
 #include <net-snmp/utilities.h>
 #include <net-snmp/library/system.h>    /* for "internal" definitions */
 
 #include <net-snmp/library/snmp_api.h>
+#include <net-snmp/library/read_config.h> /* for get_temp_file_pattern() */
 
 #ifndef IFF_LOOPBACK
 #	define IFF_LOOPBACK 0
@@ -685,10 +694,10 @@ get_boottime(void)
     struct pst_static pst_buf;
 #else
     struct timeval  boottime;
-#ifdef	CAN_USE_SYSCTL
+#ifdef	NETSNMP_CAN_USE_SYSCTL
     int             mib[2];
     size_t          len;
-#else
+#elif defined(NETSNMP_CAN_USE_NLIST)
     int             kmem;
     static struct nlist nl[] = {
 #if !defined(hpux)
@@ -698,7 +707,7 @@ get_boottime(void)
 #endif
         {(char *) ""}
     };
-#endif                          /* CAN_USE_SYSCTL */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL */
 #endif                          /* hpux10 || hpux 11 */
 
 
@@ -708,8 +717,7 @@ get_boottime(void)
 #if defined(hpux10) || defined(hpux11)
     pstat_getstatic(&pst_buf, sizeof(struct pst_static), 1, 0);
     boottime_csecs = pst_buf.boot_time * 100;
-#else
-#ifdef CAN_USE_SYSCTL
+#elif NETSNMP_CAN_USE_SYSCTL
     mib[0] = CTL_KERN;
     mib[1] = KERN_BOOTTIME;
 
@@ -717,7 +725,7 @@ get_boottime(void)
 
     sysctl(mib, 2, &boottime, &len, NULL, 0);
     boottime_csecs = (boottime.tv_sec * 100) + (boottime.tv_usec / 10000);
-#else                           /* CAN_USE_SYSCTL */
+#elif defined(NETSNMP_CAN_USE_NLIST)
     if ((kmem = open("/dev/kmem", 0)) < 0)
         return 0;
     nlist(KERNEL_LOC, nl);
@@ -730,7 +738,8 @@ get_boottime(void)
     read(kmem, &boottime, sizeof(boottime));
     close(kmem);
     boottime_csecs = (boottime.tv_sec * 100) + (boottime.tv_usec / 10000);
-#endif                          /* CAN_USE_SYSCTL */
+#else
+    return 0;
 #endif                          /* hpux10 || hpux 11 */
 
     return (boottime_csecs);
@@ -743,7 +752,7 @@ get_boottime(void)
 long
 get_uptime(void)
 {
-#if !defined(solaris2) && !defined(linux) && !defined(cygwin)
+#if !defined(solaris2) && !defined(linux) && !defined(cygwin) && !defined(aix4) && !defined(aix5)
     struct timeval  now;
     long            boottime_csecs, nowtime_csecs;
 
@@ -756,6 +765,20 @@ get_uptime(void)
     return (nowtime_csecs - boottime_csecs);
 #endif
 
+#if defined(aix4) || defined(aix5)
+    struct nlist nl;
+    int kmem;
+    time_t lbolt;
+    nl.n_name = "lbolt";
+    if(knlist(&nl, 1, sizeof(struct nlist)) != 0) return(0);
+    if(nl.n_type == 0 || nl.n_value == 0) return(0);
+    if((kmem = open("/dev/mem", 0)) < 0) return 0;
+    lseek(kmem, (long) nl.n_value, L_SET);
+    read(kmem, &lbolt, sizeof(lbolt));
+    close(kmem);
+    return(lbolt);
+#endif
+
 #ifdef solaris2
     kstat_ctl_t    *ksc = kstat_open();
     kstat_t        *ks;
@@ -990,7 +1013,7 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     char           *ourcopy = strdup(pathname);
     char           *entry;
     char            buf[SNMP_MAXPATH];
-    char           *st;
+    char           *st = NULL;
 
 #if defined (WIN32) || defined (cygwin)
     /* convert backslash to forward slash */
@@ -1055,6 +1078,46 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     return SNMPERR_SUCCESS;
 }
 
+/**
+ * netsnmp_mktemp creates a temporary file based on the
+ *                 configured tempFilePattern
+ *
+ * @return file descriptor
+ */
+const char     *
+netsnmp_mktemp(void)
+{
+    static char     name[32];
+    int             fd = -1;
+
+    strcpy(name, get_temp_file_pattern());
+#ifdef HAVE_MKSTEMP
+    fd = mkstemp(name);
+#else
+    if (mktemp(name)) {
+# ifndef WIN32
+        fd = open(name, O_CREAT | O_EXCL | O_WRONLY);
+# else
+        /*
+         * Win32 needs _S_IREAD | _S_IWRITE to set permissions on file
+         * after closing
+         */
+        fd = _open(name, _O_CREAT,
+                   _S_IREAD | _S_IWRITE | _O_EXCL | _O_WRONLY);
+# endif
+    }
+#endif
+    if (fd >= 0) {
+        close(fd);
+        DEBUGMSGTL(("netsnmp_mktemp", "temp file created: %s\n",
+                    name));
+        return name;
+    }
+    snmp_log(LOG_ERR, "netsnmp_mktemp: error creating file %s\n",
+             name);
+    return NULL;
+}
+
 /*
  * This function was created to differentiate actions
  * that are appropriate for Linux 2.4 kernels, but not later kernels.
@@ -1096,3 +1159,32 @@ static int printOSonce = 1;
 #endif /* HAVE_SYS_UTSNAME_H */
 }
 
+/**
+ * netsnmp_os_kernel_width determines kernel width at runtime
+ * Currently implemented for IRIX, AIX and Tru64 Unix
+ *
+ * @return kernel width (usually 32 or 64) on success, -1 on error
+ */
+int
+netsnmp_os_kernel_width(void)
+{
+#ifdef irix6
+  char buf[8];
+  sysinfo(_MIPS_SI_OS_NAME, buf, 7);
+  if (strncmp("IRIX64", buf, 6) == 0) {
+    return 64;
+  } else if (strncmp("IRIX", buf, 4) == 0) {
+    return 32;
+  } else {
+    return -1;
+  }
+#elif defined(aix4) || defined(aix5)
+  return (__KERNEL_32() ? 32 : (__KERNEL_64() ? 64 : -1));
+#elif defined(osf4) || defined(osf5) || defined(__alpha)
+  return 64; /* Alpha is always 64bit */
+#else
+  /* kernel width detection not implemented */
+  return -1;
+#endif
+}
+
