@@ -254,15 +254,15 @@ memdup(u_char ** to, const u_char * from, size_t size)
 char           *
 netsnmp_strdup_and_null(const u_char * from, size_t from_len)
 {
-    u_char         *ret;
+    char         *ret;
 
     if (from_len == 0 || from[from_len - 1] != '\0') {
-        ret = malloc(from_len + 1);
+        ret = (char *)malloc(from_len + 1);
         if (!ret)
             return NULL;
         ret[from_len] = '\0';
     } else {
-        ret = malloc(from_len);
+        ret = (char *)malloc(from_len);
         if (!ret)
             return NULL;
         ret[from_len - 1] = '\0';
@@ -437,7 +437,8 @@ netsnmp_hex_to_binary(u_char ** buf, size_t * buf_len, size_t * offset,
     }
 
     while (*cp != '\0') {
-        if (!isxdigit((int) *cp)) {
+        if (!isxdigit((int) *cp) ||
+            !isxdigit((int) *(cp+1))) {
             if ((NULL != delim) && (NULL != strchr(delim, *cp))) {
                 cp++;
                 continue;
@@ -585,7 +586,7 @@ dump_chunk(const char *debugtoken, const char *title, const u_char * buf,
  * FIX	Use something other than snprint_hexstring which doesn't add 
  *	trailing spaces and (sometimes embedded) newlines...
  */
-#ifdef SNMP_TESTING_CODE
+#ifdef NETSNMP_ENABLE_TESTING_CODE
 char           *
 dump_snmpEngineID(const u_char * estring, size_t * estring_len)
 {
@@ -759,7 +760,7 @@ dump_snmpEngineID(const u_char * estring, size_t * estring_len)
 
 #undef eb
 }                               /* end dump_snmpEngineID() */
-#endif                          /* SNMP_TESTING_CODE */
+#endif                          /* NETSNMP_ENABLE_TESTING_CODE */
 
 
 /**
@@ -1042,3 +1043,35 @@ char *netsnmp_getenv(const char *name)
 #endif
 }
 
+/*
+ * swap the order of an inet addr string
+ */
+int
+netsnmp_addrstr_hton(char *ptr, size_t len)
+{
+#ifndef WORDS_BIGENDIAN
+    char tmp[8];
+    
+    if (8 == len) {
+        tmp[0] = ptr[6];
+        tmp[1] = ptr[7];
+        tmp[2] = ptr[4];
+        tmp[3] = ptr[5];
+        tmp[4] = ptr[2];
+        tmp[5] = ptr[3];
+        tmp[6] = ptr[0];
+        tmp[7] = ptr[1];
+        memcpy (ptr, &tmp, 8);
+    }
+    else if (32 == len) {
+        netsnmp_addrstr_hton(ptr   , 8);
+        netsnmp_addrstr_hton(ptr+8 , 8);
+        netsnmp_addrstr_hton(ptr+16, 8);
+        netsnmp_addrstr_hton(ptr+24, 8);
+    }
+    else
+        return -1;
+#endif
+
+    return 0;
+}
